/*
 * AudioFileFilter.java	
 *
 * Copyright 2007 Cristina Roura Claver cristina.roura@gmail.com
 *
 * This file is part of LHMC.
 *
 * LHMC is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * LHMC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package lhmc.files;

import java.io.*;
import javax.swing.*;
import java.net.*;

import jcifs.smb.*;

/**
 * AudioFileFilter is an instance of the FileFilter interface
 * Accept all directories and image formats that our plugins can display.
 * 
 * @author 	Cristina Roura Claver
 * @version 	0.01 Jun 2007
 */
public class AudioFileFilter implements FileFilter, SmbFileFilter {

    private final static int LOCAL_FILE = 0;
    private final static int REMOTE_SMBFILE = 1;       
    
    private final static String mp3  =   "mp3";
    private final static String ogg  =   "ogg";
    private final static String flac =   "flac";
    private final static String ape  =   "ape";
    private final static String wav  =   "wav";
    private final static String aiff =   "aiff";
    private final static String au   =   "au";

    /**
     * Accept all directories and all mp3, ogg, flac, MONKEY's AUDIO, WAV, AIFF, AU and SPEEX audio formats.
     * @param url  The url to accept.
     * @return      True if its accepted.
     */             
    public boolean accept(URL url) {       
        boolean isAccepted = false; 
        UrlUtils urlUtils = new UrlUtils();
        int filesystemLocation = urlUtils.getLocation(url);
        if (filesystemLocation == LOCAL_FILE) {
            File file = urlUtils.getFile(url);
            isAccepted = accept(file);
        } else if (filesystemLocation == REMOTE_SMBFILE) {
            SmbFile smbfile = urlUtils.getSmbFile(url);
            isAccepted = accept(smbfile);          
        }
        return isAccepted;
    }    
    
    /**
     * Accept all directories and all mp3, ogg, flac, MONKEY's AUDIO, WAV, AIFF, AU and SPEEX audio formats.
     * @param file  The file to accept.
     * @return      True if its accepted.
     */             
    public boolean accept(File file) {
        if (file.isDirectory()) {
            return true;           
        }
        String extension = getExtension(file.getName());
        return allowedExtension(extension);
    }

    /**
     * Accept all directories and all mp3, ogg, flac, MONKEY's AUDIO, WAV, AIFF, AU and SPEEX audio formats.
     * @param file  The SmbFile to accept.
     * @return      True if its accepted.
     */             
    public boolean accept(SmbFile file) {
        try {
            if (file.isDirectory()) {
                return true;
            }
        } catch (SmbException e) {//Catch exception if any
            System.err.println("SmbException: " + e.getMessage());
        }  
        String extension = getExtension(file.getName());
        return allowedExtension(extension);
    }
    
    /**
     * Return if an extension is allowed by this filter.
     * @param extension  The string of the extension.
     * @return      true if the extension is allowed.
     */      
    private boolean allowedExtension(String extension) {
        if (extension != null) {
            if (extension.equals(mp3) ||
                extension.equals(ogg) ||
                extension.equals(flac) ||
                extension.equals(ape) ||
                extension.equals(wav) ||
                extension.equals(aiff) ||                  
                extension.equals(au)) {
                    return true;
            } else {
                return false;
            }
        }
        return false;        
    }
    
    /**
     * Gets the extension of a file.
     * @param fileName  The string of the file name to get the extension.
     * @return      The extension in a String.
     */     
    private static String getExtension(String fileName) {
        String ext = null;
        int i = fileName.lastIndexOf('.');

        if (i > 0 &&  i < fileName.length() - 1) {
            ext = fileName.substring(i+1).toLowerCase();
        }
        return ext;
    }
}