/*
 * CopyManager.java	
 *
 * Copyright 2007 Cristina Roura Claver cristina.roura@gmail.com
 *
 * This file is part of LHMC.
 *
 * LHMC is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * LHMC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package lhmc.files;

import java.io.*;
import java.net.*;
import java.util.Vector;
import java.util.Arrays;

import jcifs.smb.*;

import lhmc.utils.*;
    

/**
 * CopyManager manages the copy of files between difrent locations.
 * It can copy: from local to local.
 *              from local to remote.
 *              from remote to local.
 *              from remote to remote.
 * 
 * @author 	Cristina Roura Claver
 * @version 	0.01 Jun 2007
 */
public class CopyManager {
        
    /*
     String actualLocation  -> filesystemlocation
     String locationToCopy -> hostToSave
     String locationToCopyType ->   hostType   
     Vector<URL> filesToCopy -> filesToSave
     String localPathToCopy   -> getMediaUrl();
     String directoryToCopy    -> directory
     */  
    String actualLocation = null;
    String locationToCopy = null;
    String locationToCopyType = null;
    String localPathToCopy = null;
    String mediaType = null;
    FileFilter customFileFilter = null;
    UrlUtils urlUtils = null;    
    
    /**
     * Constructor of CopyManager
     * @param actualLocation        the location where the files to copy lay.
     * @param locationToCopy        the location where the files would be copied.
     * @param locationToCopyType    the type of the location to copy.
     * @param localPathToCopy       the path where the files would be copied.
     * @param mediaType             the type of media files to copy.
     * @param customFileFilter      the filefilter to apply.
     */          
    public CopyManager(String actualLocation, String locationToCopy, String locationToCopyType, String localPathToCopy, String mediaType, FileFilter customFileFilter) {
        this.actualLocation = actualLocation;
        this.locationToCopy = locationToCopy;
        this.locationToCopyType = locationToCopyType;
        this.localPathToCopy = localPathToCopy;
        this.mediaType = mediaType;
        this.customFileFilter = customFileFilter;
        urlUtils = new UrlUtils();
    }

    /**
     * Copies a vector of url files to a directory.
     * @param filesToCopy       the vector of url files to copy.
     * @param directoryToCopy   the directory to copy the files.
     */
    public void copyFiles(Vector<URL> filesToCopy, String directoryToCopy) {
        if (locationToCopyType == "Home") {             
            /*copy from Remote host to Local drive */       
            copyRemoteToLocal(filesToCopy, localPathToCopy, directoryToCopy, false);
            
        } else if (locationToCopyType == "Removable") {
            if ( actualLocation == "Home") {
                /*copy from local drive to removable device */
                copyLocalToLocal(filesToCopy, directoryToCopy);
            } else {
                /*copy from remote host to removable device */
                copyRemoteToLocal(filesToCopy, locationToCopy, directoryToCopy, true);
            }            
        } else if (locationToCopyType == "Remote") {
            if ( actualLocation == "Home") {
                /*copy from local drive to remote */
                copyLocalToRemote(filesToCopy, directoryToCopy);                                
            } else {
                /*copy from remote host to another remote host */
                copyRemoteToRemote(filesToCopy, directoryToCopy);                                                
            }
        }
    }
    
    /**
     * Copies a remote vector of url files to a local directory.
     * @param filesToCopy       the vector of url files to copy.
     * @param pathToCopy        the path to copy the files.
     * @param directoryToCopy   the directory to copy the files.
     * @param activateRemovables Denotes if the copy is to a removable device.
     */    
    private void copyRemoteToLocal(Vector<URL> filesToCopy, String pathToCopy, String directoryToCopy, boolean activateRemovables) {
                                   
        for (int i = 0; i < filesToCopy.size(); i++) {   
            /* creates the new directory, if it doesn't exist, to copy the files */            
            File newDirectory = getNewDirectoryFile(pathToCopy, directoryToCopy);
            
            if (urlUtils.isUrlDirectory(filesToCopy.elementAt(i))) {
                /* if the copy is from a directory */
                String newDir = directoryToCopy  + "/" + urlUtils.getUrlName(filesToCopy.elementAt(i));
                System.out.println("ImagesJPanel - saveTo() - IS DIRECTORY");                                                                    
                System.out.println("ImagesJPanel - saveTo() - NEW DIRECTORY: " + newDir);  
                /* get the contents of the directory, and puts it in a vector */                
                URL[] directoryFiles = urlUtils.getFilteredUrls(filesToCopy.elementAt(i), customFileFilter);                        
                Vector vectorFiles = new Vector(Arrays.asList(directoryFiles));
                /* do the copy of this new vector */                
                copyRemoteToLocal( vectorFiles, pathToCopy, newDir, activateRemovables);
            } else {
                /* if the copy is from a file */
                System.out.println("ImagesJPanel - saveTo() - IS NOT DIRECTORY"); 
                /* gets the remote file to copy */
                SmbFile smbFile = urlUtils.getSmbFile(filesToCopy.elementAt(i));
                /* creates the new destination file */
                File destination = new File(newDirectory.getPath() + "/" + smbFile.getName());                                                        
                System.out.println("ImagesJPanel - saveTo() - fem la copia de: " + smbFile.getPath());                    
                System.out.println("ImagesJPanel - saveTo() - fem la copia a: " + destination.getPath());  
                /* does the copy */
                CopyTo copyTo = new CopyTo(smbFile, destination);  
                if (activateRemovables) {
                    activateRemovables();
                }
                copyTo.run();
            }
        }       
    }
    
    /**
     * Copies a local vector of url files to a local directory.
     * @param filesToCopy       the vector of url files to copy.
     * @param directoryToCopy   the directory to copy the files.
     */     
    private void copyLocalToLocal(Vector<URL> filesToCopy, String directoryToCopy) {    
        for (int i = 0; i < filesToCopy.size(); i++) {                        
            /* creates the new directory, if it doesn't exist, to copy the files */
            File newDirectory = getNewDirectoryFile(locationToCopy, directoryToCopy);                        
            if (urlUtils.isUrlDirectory(filesToCopy.elementAt(i))) {
                /* if the copy is from a directory */                   
                String newDir = directoryToCopy + "/" + urlUtils.getUrlName(filesToCopy.elementAt(i));
                System.out.println("ImagesJPanel - saveTo() - IS DIRECTORY");                                                                    
                System.out.println("ImagesJPanel - saveTo() - NEW DIRECTORY: " + newDir);  
                /* get the contents of the directory, and puts it in a vector */
                URL[] directoryFiles = urlUtils.getFilteredUrls(filesToCopy.elementAt(i), customFileFilter);                        
                Vector vectorFiles = new Vector(Arrays.asList(directoryFiles));
                /* do the copy of this new vector */
                copyLocalToLocal( vectorFiles, newDir);
            } else {
                /* if the copy is from a file */                
                System.out.println("ImagesJPanel - saveTo() - IS NOT DIRECTORY");                                                  
                /* gets the local file to copy */                
                File file = urlUtils.getFile(filesToCopy.elementAt(i));
                /* creates the new destination file */                
                File destination = new File(newDirectory.getPath() + "/" + file.getName());                                                        
                System.out.println("ImagesJPanel - saveTo() - fem la copia de: " + file.getPath());                    
                System.out.println("ImagesJPanel - saveTo() - fem la copia a: " + destination.getPath());                    
                /* does the copy */                
                CopyTo copyTo = new CopyTo(file, destination);   
                activateRemovables();
                copyTo.run();
            }
        }
    }   
    
    /**
     * Copies a local vector of url files to a remote directory.
     * @param filesToCopy       the vector of url files to copy.
     * @param directoryToCopy   the directory to copy the files.
     */      
    private void copyLocalToRemote(Vector<URL> filesToCopy, String directoryToCopy) {    
        try {            
            for (int i = 0; i < filesToCopy.size(); i++) {   
                /* creates the new directory, if it doesn't exist, to copy the files */                
                SmbFile newDirectory = getNewDirectorySmbFile(locationToCopy, directoryToCopy);                              
                System.out.println("ImagesJPanel - saveTo(): " + urlUtils.getPath(filesToCopy.elementAt(i)));                                                                        
                if (urlUtils.isUrlDirectory(filesToCopy.elementAt(i))) {
                    /* if the copy is from a directory */                                       
                    String newDir = directoryToCopy + "/" + urlUtils.getUrlName(filesToCopy.elementAt(i));
                    System.out.println("ImagesJPanel - saveTo() - IS DIRECTORY");                                                                    
                    System.out.println("ImagesJPanel - saveTo() - NEW DIRECTORY: " + newDir);   
                    /* get the contents of the directory, and puts it in a vector */                       
                    URL[] directoryFiles = urlUtils.getFilteredUrls(filesToCopy.elementAt(i), customFileFilter);                        
                    Vector vectorFiles = new Vector(Arrays.asList(directoryFiles));
                    copyLocalToRemote( vectorFiles, newDir);                            
                } else {
                    /* if the copy is from a file */                
                    System.out.println("ImagesJPanel - saveTo() - IS NOT DIRECTORY");                               
                    /* gets the local file to copy */                                    
                    File file = urlUtils.getFile(filesToCopy.elementAt(i));
                    /* creates the new destination file */                    
                    SmbFile destination = new SmbFile(newDirectory.getPath() + "/" + file.getName());                                                        
                    System.out.println("ImagesJPanel - saveTo() - fem la copia de: " + file.getPath());                    
                    System.out.println("ImagesJPanel - saveTo() - fem la copia a: " + destination.getPath());                    
                    /* does the copy */                                    
                    CopyTo copyTo = new CopyTo(file, destination);       
                    copyTo.run();                                
                }                                                            
            }                      
        } catch (MalformedURLException ex) {
            System.err.println("MalformedURLException: " + ex.getMessage());
        }
    }
    
    /**
     * Copies a remote vector of url files to a remote directory.
     * @param filesToCopy       the vector of url files to copy.
     * @param directoryToCopy   the directory to copy the files.
     */       
    private void copyRemoteToRemote(Vector<URL> filesToCopy, String directoryToCopy) {                
        try {
            /* creates the new directory, if it doesn't exist, to copy the files */                
            SmbFile newDirectory = getNewDirectorySmbFile(locationToCopy, directoryToCopy);             
            for (int i = 0; i < filesToCopy.size(); i++) {
                /* gets the remote file to copy */                                                    
                SmbFile file = urlUtils.getSmbFile(filesToCopy.elementAt(i));
                /* creates the new destination file */                                    
                SmbFile destination = new SmbFile(newDirectory.getPath() + "/" + file.getName());                                                        
                System.out.println("ImagesJPanel - saveTo() - fem la copia de: " + file.getPath());                    
                System.out.println("ImagesJPanel - saveTo() - fem la copia a: " + destination.getPath());   
                /* does the copy */                                                    
                CopyTo copyTo = new CopyTo(file, destination);       
                copyTo.run();
            }
        } catch (MalformedURLException ex) {
            System.err.println("MalformedURLException: " + ex.getMessage());
        }        
    }
    
    /**
     * Creates the new directory File to copy.
     * @param pathToCopy        the path to copy the files.
     * @param directoryToCopy   the directory to copy the files.
     * @return                  the new directory File.
     */   
    private File getNewDirectoryFile(String pathToCopy, String directoryToCopy) {
        File localPath = new File(pathToCopy);                    
        File directory = new File(localPath.getPath() + "/" + directoryToCopy); 
        if (!directory.isDirectory()) {
            directory.mkdir();               
        }
        System.out.println("ImagesJPanel - getNewDirectoryFile() - nou directory: " + directory.getPath());           
        return directory;
    }    

    /**
     * Creates the new directory SmbFile to copy.
     * @param pathToCopy        the path to copy the files.
     * @param directoryToCopy   the directory to copy the files.
     * @return                  the new directory SmbFile.
     */       
    private SmbFile getNewDirectorySmbFile(String pathToCopy, String directoryToCopy) {
        SmbFile directory = null;
        try {
            SmbFile remotePath = new SmbFile("smb://" + pathToCopy + mediaType);
            directory = new SmbFile(remotePath.getPath() + directoryToCopy);
            if (!directory.isDirectory()) {
                directory.mkdir();               
            }
        } catch (SmbException ex) {
            System.err.println("SmbException: " + ex.getMessage());
        } catch (MalformedURLException ex) {
            System.err.println("MalformedURLException: " + ex.getMessage());
        }   
        return directory;
    }                  

    /**
     * Activates the removable devices connected to the system.
     */      
    private void activateRemovables() {
        String pluginUrl = null;       
        URL root = null;
        ConfigReader config = new ConfigReader();        
        pluginUrl = config.getProperty("URLRemovableDev");
        try {
            root =  new URL("file://" + pluginUrl);
        } catch (MalformedURLException e) {
            System.out.println("MalformedURLException: " + e);   
        }
        RemovableDevicesActivator removableDevicesActivator = new RemovableDevicesActivator(root);                 
    }    
}
