/*
 * CopyTo.java	
 *
 * Copyright 2007 Cristina Roura Claver cristina.roura@gmail.com
 *
 * This file is part of LHMC.
 *
 * LHMC is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * LHMC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package lhmc.files;

import java.io.*;
import java.net.*;

import jcifs.smb.*;


/**
 * CopyTo is a threaded class that manages the copy of files from remote 
 * filesystem (SmbFile) to local filesystem (File) and vice versa. 
 * Also manages to copy from remote to remote filesystem or local to local
 * filesystem.
 * 
 * @author 	Cristina Roura Claver
 * @version 	0.01 20 Jun 2007
 */
public class CopyTo extends Thread {
    
    /* streams used to copy the files */
    private InputStream inStream = null;
    private OutputStream outStream = null;
    
    /* smbfiles used only to copy to and from remote filesystems */
    private SmbFile source = null;
    private SmbFile destination = null;
    
    /**
     * Constructor that creates the input and output stream from source file
     * and destination file.
     * @param srcFile the source file.
     * @param destFile the destination file.
     */       
    public CopyTo(File srcFile, File destFile) {     
        try {
            inStream = new FileInputStream(srcFile);
            outStream = new FileOutputStream(destFile);     
        } catch (FileNotFoundException e) {
            System.err.println("FileNotFoundException: " + e.getMessage());
        }
    }
    
    /**
     * Constructor that creates the input and output stream from source file
     * and destination smbfile.
     * @param srcFile the source file.
     * @param destFile the destination smbfile.
     * @see SmbFile
     */     
    public CopyTo(File srcFile, SmbFile destFile) {
        try {
            inStream = new FileInputStream(srcFile);
            outStream = new SmbFileOutputStream(destFile);        
        } catch (FileNotFoundException e) {
            System.err.println("FileNotFoundException: " + e.getMessage());
        } catch (SmbException e) {
            System.err.println("SmbException: " + e.getMessage());
        } catch (MalformedURLException e) {
            System.err.println("MalformedURLException: " + e.getMessage());
        } catch (UnknownHostException e) {
            System.err.println("UnknownHostException: " + e.getMessage());            
        }                
    }

    /**
     * Constructor that creates the source and destination smbfile.
     * @param srcFile the source smbfile.
     * @param destFile the destination smbfile.
     * @see SmbFile
     */         
    public CopyTo(SmbFile srcFile, SmbFile destFile) {
        source = srcFile;
        destination = destFile;
    }
    
    /**
     * Constructor that creates the input and output stream from source smbfile
     * and destination file.
     * @param srcFile the source smbfile.
     * @param destFile the destination file.
     * @see SmbFile
     */        
    public CopyTo(SmbFile srcFile, File destFile) {
        try {
            inStream = new SmbFileInputStream(srcFile);
            outStream = new FileOutputStream(destFile);        
        } catch (FileNotFoundException e) {
            System.err.println("FileNotFoundException: " + e.getMessage());
        } catch (SmbException e) {
            System.err.println("SmbException: " + e.getMessage());
        } catch (MalformedURLException e) {
            System.err.println("MalformedURLException: " + e.getMessage());
        } catch (UnknownHostException e) {
            System.err.println("UnknownHostException: " + e.getMessage());            
        }
    }   
    
    /**
     * Executes the copy of the files.
     */           
    public void run() {
        if (source != null && destination != null ) {
            try {
                /* if the copy is beetween smbfiles, we can use the method from smbfile copyTo */
                source.copyTo(destination);
            } catch (SmbException e) {
                System.err.println("SmbException: " + e.getMessage());
            }            
        } else {
            copyStreams(inStream, outStream);
        }
    }    
    
    /**
     * Makes the copy of an inputstream to an outputstream.
     * @param in    the inputstream.
     * @param out   the outputstream.
     */       
    private void copyStreams(InputStream in, OutputStream out) {
        int bufferSize = 4 * 1024;
        byte[] buffer = new byte[bufferSize];
        int bytesRead;
        try {
            while ((bytesRead = in.read(buffer)) >= 0) {
                out.write(buffer, 0, bytesRead);
            }
            out.close();
            in.close();
        } catch (IOException e) {
            System.err.println("IOException: " + e.getMessage());            
        }         
    }
}