/*
 * ImageFileFilter.java	
 *
 * Copyright 2007 Cristina Roura Claver cristina.roura@gmail.com
 *
 * This file is part of LHMC.
 *
 * LHMC is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * LHMC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package lhmc.files;

import java.io.*;
import javax.swing.*;
import java.net.*;

import jcifs.smb.*;

/**
 * ImageFileFilter is an instance of the FileFilter interface
 * Aceept all directories and image formats that our plugins can display.
 * 
 * @author 	Cristina Roura Claver
 * @version 	0.01 Jul 2007
 */
public class ImageFileFilter implements FileFilter, SmbFileFilter {
    
    private final static int LOCAL_FILE = 0;
    private final static int REMOTE_SMBFILE = 1;        
    
    private final static String bmp  =   "bmp";
    private final static String gif  =   "gif";
    private final static String jpeg =   "jpeg";
    private final static String jpg  =   "jpg";
    private final static String png  =   "png";   
    private final static String wbmp  =   "wbmp";
       
    /**
     * Accept all directories and all bmp, gif, jpeg, jpg, png or wbmp files. 
     * @param url  The URL to accept.
     * @return      True if its accepted.
     */             
    public boolean accept(URL url) {
        boolean isAccepted = false; 
        UrlUtils urlUtils = new UrlUtils();
        int filesystemLocation = urlUtils.getLocation(url);
        if (filesystemLocation == LOCAL_FILE) {
            File file = urlUtils.getFile(url);
            isAccepted = accept(file);
        } else if (filesystemLocation == REMOTE_SMBFILE) {
            SmbFile smbfile = urlUtils.getSmbFile(url);
            isAccepted = accept(smbfile);          
        }
        return isAccepted;
    }
    
    /**
     * Accept all directories and all bmp, gif, jpeg, jpg, png or wbmp files. 
     * @param file  The file to accept.
     * @return      True if its accepted.
     */             
    public boolean accept(File file) {
        if (file.isDirectory()) {
            return true;
        }
        String extension = getExtension(file.getName());
        return allowedExtension(extension);
    }

    /**
     * Accept all directories and all bmp, gif, jpeg, jpg, png or wbmp files. 
     * @param file  The smbfile to accept.
     * @return      True if its accepted.
     */             
    public boolean accept(SmbFile file) {
        try {
            if (file.isDirectory()) {
                return true;
            }
        } catch (SmbException e) {//Catch exception if any
            System.err.println("SmbException: " + e.getMessage());
        }  
        String extension = getExtension(file.getName());
        return allowedExtension(extension);
    }
    
    /**
     * Return if an extension is allowed by this filter.
     * @param extension  The string of the extension.
     * @return      true if the extension is allowed.
     */         
    private boolean allowedExtension(String extension) {
        if (extension != null) {
            if (extension.equals(bmp) ||
                extension.equals(gif) ||
                extension.equals(jpeg) ||
                extension.equals(jpg) ||
                extension.equals(png) ||
                extension.equals(wbmp)) {
                    return true;
            } else {
                return false;
            }
        }
        return false;      
    }
    
    /**
     * Gets the extension of a file.
     * @param fileName  The string of the file name to get the extension.
     * @return      The extension in a String.
     */     
    private static String getExtension(String fileName) {
        String ext = null;
        int i = fileName.lastIndexOf('.');

        if (i > 0 &&  i < fileName.length() - 1) {
            ext = fileName.substring(i+1).toLowerCase();
        }
        return ext;
    }
}