/* SortedTreeNode.java
 *
 * 02.08.06 - new class (previously inner class of Niffler)
 *
 */

/*
 * Niffler Image Viewer
 * Copyright (c) 2003-2006, Norman Hendrich
 * All rights reserved.
 *
 * Contact:
 *
 * Norman Hendrich
 * c/o University of Hamburg
 * FB Informatik
 * Vogt-Koelln-Str. 30
 * D 22527 Hamburg
 * Germany
 *
 * hendrich@informatik.uni-hamburg.de
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions 
 * are met:
 *
 *   * Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above copyright notice, 
 *     this list of conditions and the following disclaimer in the documentation 
 *     and/or other materials provided with the distribution.
 *   * Neither the name of the University of Hamburg nor the names of its 
 *     contributors may be used to endorse or promote products derived from 
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE 
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER 
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, 
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * Modified by: Cristina Roura Claver cristina.roura@gmail.com
 * Jun 2007
 * 
 * CHANGES:
 *      Package name modified.
 *      Changed isLeaf() for getAllowsChildren() and setAllowsChildren() to distinguish directories from files.
 *      Javadoc Comments added
 *      Support to URL
 * 
 */

package lhmc.files;

import javax.swing.tree.*;
import java.io.File;
import java.net.*;

import jcifs.smb.*;
 
/**  
 * SortedTreeNode: a mutable tree node with alphabetically sorted children
 */
public class SortedTreeNode extends  DefaultMutableTreeNode {
    
    static int FILE_TYPE = 0;
    static int SMBFILE_TYPE = 1;
    int urlType = -1;
        
    private ImageFileFilter imageFileFilter = new ImageFileFilter();
    private AudioFileFilter audioFileFilter = new AudioFileFilter();
    private VideoFileFilter videoFileFilter = new VideoFileFilter();
    
    int index;
    

    /**
     * Constructs a SortedTreeNode, specifying the filesystem location.
     * @param fileType  remote or local file type
     */    
    public SortedTreeNode(int fileType) {
        super();
        index = -1;
        urlType = fileType;      
    }

    /**
     * Constructs a SortedTreeNode with a root node as an argument.
     * @param userObject    the object root node.
     */      
    public SortedTreeNode( Object userObject) {
      super( userObject );      
    }


    /**
     * Gets the index of the sortedTreeNode.
     * @return  the index of the treeNode.
     */
    public int getIndex() { 
        return index; 
    }
    
    /**
     * Gets the index of the sortedTreeNode.
     * @param index the index of the treeNode.
     */    
    public void setIndex( int index ) { 
        this.index = index; 
    }
    

    /** 
     * Adds a TreeNode to the child list.
     * @param child     the SortedTreeNode to add.
     */
    public synchronized void add( SortedTreeNode child ) {
        // System.out.println( "-#- add: " + this + " " + child );
        
        /* get name of new SortedTreeNode */
        String name = child.toString();   
     
        URL urlFile = null;
      
        try {
            urlFile = new URL(name);
        } catch (MalformedURLException e) {
            System.out.println("SortedTreeNode - add()");               
            System.err.println("MalformedURLException: " + e.getMessage());            
        }          
      
        /* Determine if it is a directory */
        if (urlType == FILE_TYPE) {
            File file = new File(urlFile.getPath());
            if (!file.isDirectory()) {          
                child.setAllowsChildren(false);
            } else {          
                child.setAllowsChildren(true);
            }
        } else if (urlType == SMBFILE_TYPE) {
            try {
                SmbFile file = new SmbFile("smb:/" + urlFile.getPath());
                if (!file.isDirectory()) {          
                    child.setAllowsChildren(false);
                } else {          
                    child.setAllowsChildren(true);
                }   
            } catch (MalformedURLException e) {
                System.out.println("SortedTreeNode - add()");               
                System.err.println("MalformedURLException: " + e.getMessage());            
            } catch (SmbException e) {
                System.err.println("SmbException: " + e.getMessage());                          
            }
        }
       
        if (getChildCount() == 0) { // first child
            super.add( child ); 
            child.setParent(this);
            //System.out.println( "SortedTreeNode add - First: " +  name ); 
            return;
        }

        child.setParent(this);

        name = child.toString().toLowerCase();
      
        int i = 0;
        //if (child.isLeaf()) {
        if (!child.getAllowsChildren()) { /* insert leaf node */    
            //System.out.println( "SortedTreeNode add - Leaf: " +  name ); 
            /* skip directories */                    
            while ( (i<getChildCount()) && (((SortedTreeNode)children.elementAt(i)).getAllowsChildren())) { 
                i++; 
            }

            try {
                if (audioFileFilter.accept(urlFile)) {
                    /* skip image files */
                    while ( (i<getChildCount()) && (imageFileFilter.accept(new URL(((SortedTreeNode)children.elementAt(i)).toString())))) { 
                        i++; 
                    }               
                } else if (videoFileFilter.accept(urlFile)) {
                    /* skip image and audio files */
                    while ( (i<getChildCount()) && ( imageFileFilter.accept(new URL(((SortedTreeNode)children.elementAt(i)).toString()))
                                                || audioFileFilter.accept(new URL(((SortedTreeNode)children.elementAt(i)).toString()))) ) { 
                        i++; 
                    }       
                }
            } catch (MalformedURLException e) {
                System.out.println("SortedTreeNode - add()");               
                System.err.println("MalformedURLException: " + e.getMessage());            
            }
           
            while ( (i<children.size()) &&(name.compareTo(children.elementAt(i).toString().toLowerCase())>=0)) { 
                i++; 
            }
            children.insertElementAt(child,i);
        }
        else { /* insert directory */
            //System.out.println( "SortedTreeNode add - Not Leaf: " +  name ); 
            while ( (i<children.size()) && (name.compareTo(children.elementAt(i).toString().toLowerCase())>=0)
                                        && (((SortedTreeNode)children.elementAt(i)).getAllowsChildren()) ) { 
                i++; 
            }
            children.insertElementAt(child,i);
        }
    }
}