/*
 * UrlUtils.java	
 *
 * Copyright 2007 Cristina Roura Claver cristina.roura@gmail.com
 *
 * This file is part of LHMC.
 *
 * LHMC is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * LHMC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package lhmc.files;

import java.io.*;
import java.net.*;

import jcifs.smb.*;


/**
 * UrlUtils helps the url files that can contain a File or a SmbFile.
 * 
 * @author 	Cristina Roura Claver
 * @version 	0.01 Jun 2007
 */
public class UrlUtils {
    
    static int LOCAL_FILE = 0;
    static int REMOTE_SMBFILE = 1;    
    
    int actualFilesystemLocation = -1;
    
    /**
     * Constructs the UrlUtils class.
     */        
    public UrlUtils() { }
    
    /**
     * Constructs the UrlUtils class specifying the filesystem location.
     * @param filesystemLocation    the filesystem location.
     */      
    public UrlUtils(int filesystemLocation) {
        actualFilesystemLocation = filesystemLocation;
    }

    /**
     * Constructs the UrlUtils and searches wich filesystem location has the URL.
     * @param url    the url file.
     */       
    public UrlUtils(URL url) {
        jcifs.Config.registerSmbURLHandler();
        String protocol = url.getProtocol();
        if (protocol.equals("file")) {
            actualFilesystemLocation = LOCAL_FILE;
        } else if (protocol.equals("smb")) {
            actualFilesystemLocation = REMOTE_SMBFILE;
        }
    }    
        
    /**
     * Gets the name of the file.
     * @param url    the url file.
     * @return      the name of the file.
     */    
    public String getUrlName(URL url) {
        getLocation(url);
        String name = null;
        if (actualFilesystemLocation == LOCAL_FILE) {
            File file = new File(url.getPath());
            name = file.getName();
        } else if (actualFilesystemLocation == REMOTE_SMBFILE) {
            try {            
                SmbFile smbFile = new SmbFile("smb:/" + url.getPath());         
                name = smbFile.getName();
            } catch(MalformedURLException e) {
                System.out.println("ThumbnailsBrowser - getUrlName()");
                System.out.println("MalformedURLException " + e);
            }                      
        }
        return name;
    }

    /**
     * Gets the location (remote or local) of the file.
     * @param url    the url file.
     * @return      the location.
     */     
    public int getLocation(URL url) {
        jcifs.Config.registerSmbURLHandler();
        String protocol = url.getProtocol();
        if (protocol.equals("file")) {
            actualFilesystemLocation = LOCAL_FILE;
        } else if (protocol.equals("smb")) {
            actualFilesystemLocation = REMOTE_SMBFILE;
        }
        return actualFilesystemLocation;
    }   
    
    /**
     * Gets the location attribute (remote or local).
     * @return      the location.
     */       
    public int getLocation() {
        return actualFilesystemLocation;
    }      
    
    /**
     * Tests whether the file is a directory.
     * @return      true if it is a directory.
     */      
    public boolean isUrlDirectory(URL url) {
        getLocation(url);
        boolean isDirectory = false;
        
        if (actualFilesystemLocation == LOCAL_FILE) {
            File file = new File(url.getPath());
            isDirectory = file.isDirectory();
        } else if (actualFilesystemLocation == REMOTE_SMBFILE) {
            try {
                SmbFile smbFile = new SmbFile("smb:/" + url.getPath());            
                isDirectory = smbFile.isDirectory();
            } catch(MalformedURLException e) {
                System.out.println("ThumbnailsBrowser - isUrlDirectory()");
                System.out.println("MalformedURLException " + e);
            } catch(SmbException e) {
                System.out.println("SmbException " + e);
            }       
        }
        return isDirectory;
    }
    
    /**
     * Extracts the SmbFile of a URL.
     * @param url   the url file.
     * @return  the SmbFile.
     */       
    public SmbFile getSmbFile(URL url) {
        SmbFile smbFile = null;
        try {
            smbFile = new SmbFile("smb:/" + url.getPath());
        } catch(MalformedURLException e) {
                System.out.println("ThumbnailsBrowser - getSmbFile()");
                System.out.println("MalformedURLException " + e);
            }
        return  smbFile;
    }
    
    /**
     * Extracts the File of a URL.
     * @param url   the url file.
     * @return  the File.
     */      
    public File getFile(URL url) {
        return new File(url.getPath());        
    }
    
    /**
     * Returns the pathname string of this abstract pathname's parent, or null if this pathname does not name a parent directory.
     * @param url   the url file.
     * @return  the parent url file.
     */      
    public URL getURLParent(URL url) {
        getLocation(url);
        //System.out.println("ThumbnailsBrowser - getURLParent() - url: " + url.getPath());        
        String parent = null;
        if (actualFilesystemLocation == LOCAL_FILE) {
            File file = new File(url.getPath());
            parent = file.getParent() + "/";
        } else if (actualFilesystemLocation == REMOTE_SMBFILE) {
            try {
                SmbFile smbFile = new SmbFile("smb:/" + url.getPath());            
                parent = smbFile.getParent();
           } catch(MalformedURLException e) {
               System.out.println("ThumbnailsBrowser - getURLParent()");
                System.out.println("MalformedURLException " + e);
            }   
        }
        
        return createURL(parent);
    }
    
    /**
     * Returns the absolute pathname string of this abstract pathname.
     * @param url   the url file.
     * @return  the absolute path name string.
     */         
    public String getAbsolutePath(URL url) {
        getLocation(url);
        String name = null;
        if (actualFilesystemLocation == LOCAL_FILE) {
            File file = new File(url.getPath());
            name = file.getAbsolutePath();
        } else if (actualFilesystemLocation == REMOTE_SMBFILE) {
            try {            
                SmbFile smbFile = new SmbFile("smb:/" + url.getPath());         
                name = smbFile.getPath();
            } catch(MalformedURLException e) {
                System.out.println("ThumbnailsBrowser - getUrlName()");
                System.out.println("MalformedURLException " + e);
            }                      
        }
        return name;                
    }
    
     /**
     * Converts the url file abstract pathname into a pathname string.
     * @param url   the url file.
     * @return  The string form of this abstract pathname.
     */     
    public String getPath(URL url) {
        getLocation(url);
        String name = null;
        if (actualFilesystemLocation == LOCAL_FILE) {
            File file = new File(url.getPath());
            name = file.getPath() + "/";
        } else if (actualFilesystemLocation == REMOTE_SMBFILE) {
            try {            
                SmbFile smbFile = new SmbFile("smb:/" + url.getPath());         
                name = smbFile.getPath();
            } catch(MalformedURLException e) {
                System.out.println("ThumbnailsBrowser - getUrlName()");
                System.out.println("MalformedURLException " + e);
            }                      
        }
        return name;                
    }    
   
     /**
     * Returns an array of url files and directories that satisfy the specified filter.
     * @param rootFile          the url directory to list.
     * @param customFileFilter  the file filter.     
     * @return  an array of url files.
     */        
    public URL[] getFilteredUrls(URL rootFile, FileFilter customFileFilter) {
        URL[] urls = null;
        if (actualFilesystemLocation == LOCAL_FILE) {
            File root = new File(rootFile.getPath());
            File[] files = root.listFiles(customFileFilter);        
            urls = getUrls(files);
        } else if (actualFilesystemLocation == REMOTE_SMBFILE) {
            try {
                SmbFile root = new SmbFile("smb:/" + rootFile.getPath());            
                SmbFile[] smbFiles = root.listFiles((SmbFileFilter)customFileFilter);
                urls = getUrls(smbFiles);
            } catch(MalformedURLException e) {
               System.out.println("ThumbnailsBrowser - getFilteredUrls()");
                System.out.println("MalformedURLException " + e);
            } catch(SmbException e) {
                System.out.println("SmbException " + e);
            }                        
        }
        return urls;
    }
    
     /**
     * Creates an URL file from a file path
     * @param path  the file path.     
     * @return  an url file.
     */      
    public URL createURL(String path) {
        //System.out.println("ThumbnailsBrowser - createURL() - path: " + path);                    
        URL url = null;
        try {        
            if (actualFilesystemLocation == LOCAL_FILE) {
                url = new URL("file://" + path );
            } else if (actualFilesystemLocation == REMOTE_SMBFILE) {
                String urlPath = path.substring(6);
                urlPath = "smb:///" + urlPath;
                url = new URL( urlPath );
            }        
        } catch(MalformedURLException e) {
               System.out.println("ThumbnailsBrowser - createURL()");            
            System.out.println("MalformedURLException " + e);
       }            
        return url;
    }            

     /**
     * Creates an URL array files from an array of File files.
     * @param files  the array of File files.
     * @return  an URL array.
     */     
    private URL[] getUrls(File[] files) {
        URL[] urls = new URL[files.length];
        for (int i = 0; i < files.length; i++) {
            //System.out.println("ThumbnailsBrowser - getUrls() FILE - createURL(): ");
            urls[i] = createURL(files[i].getPath());
        }   
        return urls;           
    }
    
     /**
     * Creates an URL array files from an array of SmbFile files.
     * @param smbFiles  the array of SmbFile files.
     * @return  an URL array.
     */     
    private URL[] getUrls(SmbFile[] smbFiles) {
        URL[] urls = new URL[smbFiles.length];
        for (int i = 0; i < smbFiles.length; i++) {
            //System.out.println("ThumbnailsBrowser - getUrls() SMB- createURL(): ");
            urls[i] = createURL(smbFiles[i].getPath());
        }
        return urls;         
    }
}