/*
 * VideoFileFilter.java	
 *
 * Copyright 2007 Cristina Roura Claver cristina.roura@gmail.com
 *
 * This file is part of LHMC.
 *
 * LHMC is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * LHMC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package lhmc.files;

import java.io.*;
import javax.swing.*;
import java.net.*;

import jcifs.smb.*;

/**
 * VideoFileFilter is an instance of the FileFilter interface
 * Aceept all directories and image formats that our plugins can display.
 * 
 * @author 	Cristina Roura Claver
 * @version 	0.01 Jun 2007
 */
public class VideoFileFilter implements FileFilter, SmbFileFilter {
    
    private final static int LOCAL_FILE = 0;
    private final static int REMOTE_SMBFILE = 1;    
    
    private final static String mpg  =   "mpg";
    private final static String mpeg  =   "mpeg";
    private final static String m1v =   "m1v";
    private final static String bin  =   "bin"; /* VCD/SVCD Images */
    private final static String vob  =   "vob";
    private final static String avi =   "avi";
    private final static String divx   =   "divx";
    private final static String mov   =   "mov";    
    private final static String qt   =   "qt";
    private final static String asf   =   "asf";
    private final static String viv   =   "viv";
    private final static String rm   =   "rm";
    private final static String wmv   =   "wmv";
    private final static String ogm   =   "ogm";
    private final static String mkv   =   "mkv";
    private final static String fli   =   "fli";
    private final static String flc   =   "flc";
    private final static String mvv   =   "mvv";
            
    /**
     * Accept all directories and all mpg, mpeg, m1v, bin(VCD/SVCD Images), vob, avi, divx, mov, qt, asf, viv, rm, wmv, ogm, mkv, fli, flc and mvv formats.
     * This are all formats suported by Mplayer.
     * @param url  The URL to accept.
     * @return      True if its accepted.
     */             
    public boolean accept(URL url) {
        boolean isAccepted = false; 
        UrlUtils urlUtils = new UrlUtils();
        int filesystemLocation = urlUtils.getLocation(url);
        if (filesystemLocation == LOCAL_FILE) {
            File file = urlUtils.getFile(url);
            isAccepted = accept(file);
        } else if (filesystemLocation == REMOTE_SMBFILE) {
            SmbFile smbfile = urlUtils.getSmbFile(url);
            isAccepted = accept(smbfile);          
        }
        return isAccepted;
    }
    
    /**
     * Accept all directories and all mpg, mpeg, m1v, bin(VCD/SVCD Images), vob, avi, divx, mov, qt, asf, viv, rm, wmv, ogm, mkv, fli, flc and mvv formats.
     * This are all formats suported by Mplayer.
     * @param file  The file to accept.
     * @return      True if its accepted.
     */             
    public boolean accept(File file) {
        if (file.isDirectory()) {
            return true;
        }
        String extension = getExtension(file.getName());
        return allowedExtension(extension);
    }

    /**
     * Accept all directories and all mpg, mpeg, m1v, bin(VCD/SVCD Images), vob, avi, divx, mov, qt, asf, viv, rm, wmv, ogm, mkv, fli, flc and mvv formats.
     * This are all formats suported by Mplayer.
     * @param file  The smbfile to accept.
     * @return      True if its accepted.
     */             
    public boolean accept(SmbFile file) {
        try {
            if (file.isDirectory()) {
                return true;
            }
        } catch (SmbException e) { //Catch exception if any
            System.err.println("SmbException: " + e.getMessage());
        }  
        String extension = getExtension(file.getName());
        return allowedExtension(extension);
    }
    
    /**
     * Return if an extension is allowed by this filter.
     * @param extension  The string of the extension.
     * @return      true if the extension is allowed.
     */         
    private boolean allowedExtension(String extension) {
        if (extension != null) {
            if (extension.equals(mpg) ||
                extension.equals(mpeg) ||
                extension.equals(m1v) ||
                extension.equals(bin) ||
                extension.equals(vob) ||
                extension.equals(avi) ||    
                extension.equals(divx) ||
                extension.equals(mov) ||
                extension.equals(qt) ||
                extension.equals(asf) ||
                extension.equals(viv) ||
                extension.equals(rm) ||
                extension.equals(wmv) ||
                extension.equals(ogm) ||
                extension.equals(mkv) ||
                extension.equals(fli) ||
                extension.equals(flc) ||                                        
                extension.equals(mvv)) {
                    return true;
            } else {
                return false;
            }
        }
        return false;    
    }
    
    /**
     * Gets the extension of a file.
     * @param fileName  The string of the file name to get the extension.
     * @return      The extension in a String.
     */     
    private static String getExtension(String fileName) {
        String ext = null;
        int i = fileName.lastIndexOf('.');

        if (i > 0 &&  i < fileName.length() - 1) {
            ext = fileName.substring(i+1).toLowerCase();
        }
        return ext;
    }
}