/*
 * LHMCGUI.java
 *
 * Copyright 2007 Cristina Roura Claver cristina.roura@gmail.com
 *
 * This file is part of LHMC.
 *
 * LHMC is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * LHMC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package lhmc.gui;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.text.DateFormat;
import java.util.Date;
import java.util.*;
import java.io.*;

import lhmc.utils.*;
import lhmc.plugins.*;

/**
 * Main user interface. 
 * It contains:
 * At top: A header, with the application name and date information.
 * At center: A center panel with a title and a main panel.
 * 
 * @author 	Cristina Roura Claver
 * @version 	0.01 Jan 2007
 */
public class LHMCGUI extends JFrame {

   
    /* Font types and colors */
    private Font titleFont = null;
    private Font headerFont  = null;
    private Color foregroundColor = null;

    /* Time label */
    private JLabel labelTime = new JLabel();
      
    /* Title label that changes for each mainPanel */
    private JLabel titleLabel = null;
    
    /* Main screen panel and its CardLayout  */ 
    private JPanel mainScreenCard = null;
    private CardLayout mainScreenPanelLayout = null;
    
    /* Main panel and its CardLayout  */ 
    private JPanel mainPanel = null;
    private CardLayout mainPanelLayout = null;      
            
    /* Main menu */
    private MainMenu menu = null;
    
    /* Center panel */
    private JPanel centerPanel = null;
    
    /* Recording Dvbt issues */
    private boolean isRecordingDvbt = false;
    private ExternalPlugin recordingThread = null;
    
    /**
     * Constructs LHMC GUI
     */
    public LHMCGUI() {

        /* loads the colors and fots from config file */
        loadColorFontTheme();    
	
        /* registers the keyboard keys */
        registerKeys();
        
        /* set the window title */
        setTitle("LHMC Main");
	setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

	/* creates header */
	JPanel header = createHeader(); 	

        /* creates center Panel with titlePanel and mainPanel */
        centerPanel = new JPanel(new BorderLayout());
        
        /* creates and add titlePanel */
        JPanel titlePanel = new JPanel();
        titleLabel = new JLabel("");
        titleLabel.setFont(titleFont);        
        titlePanel.add(titleLabel);        
        centerPanel.add(titlePanel, BorderLayout.PAGE_START);
        
	/* creates and show mainPanel */
	mainPanelLayout = new CardLayout();
        mainPanel = new JPanel(mainPanelLayout);
	menu = new MainMenu(this);
	mainPanel.add(menu, "MENU");
	mainPanelLayout.show(mainPanel, "MENU");       
        centerPanel.add(mainPanel,BorderLayout.CENTER);
        
	/* creates and show mainScreen composed of header and centerPanel */
        JPanel mainScreenPanel = new JPanel(new BorderLayout());
        mainScreenPanel.add(header, BorderLayout.PAGE_START);
        mainScreenPanel.add(centerPanel, BorderLayout.CENTER);
        
	mainScreenPanelLayout = new CardLayout();
        mainScreenCard = new JPanel(mainScreenPanelLayout);        
        mainScreenCard.add(mainScreenPanel, "INITIAL");
        mainScreenPanelLayout.show(mainScreenCard, "INITIAL");   
        
        getContentPane().add(mainScreenCard, BorderLayout.CENTER);
        
	/* switching to fullscreen mode if it can */
	GraphicsDevice device = GraphicsEnvironment.getLocalGraphicsEnvironment().getDefaultScreenDevice();
        boolean isFullScreen = device.isFullScreenSupported();
        setUndecorated(isFullScreen);
        setResizable(!isFullScreen);
        if (isFullScreen) {           
	    /* Full-screen mode */
            device.setFullScreenWindow(this);
            validate();
        } else {
	    /* Windowed mode */
            pack();
            setVisible(true);
        }
    }

    /**
     * Creates the header.
     * It has the application title and the current date and time.
     * @return      the header panel.
     */
    private JPanel createHeader() {
	JPanel pane = new JPanel();
	
	pane.setLayout(new BoxLayout(pane, BoxLayout.LINE_AXIS));

        /* creates and sets the application title */
	JLabel limece = new JLabel("LHMC - v0.01");
	limece.setFont(headerFont);
	limece.setForeground(foregroundColor);
	pane.add(limece);
	pane.add(Box.createHorizontalGlue());

        /* creates and sets the time */
      	labelTime.setText(updateTime());
      	javax.swing.Timer mySecTimer = new javax.swing.Timer(1000, new ActionListener() {
            public void actionPerformed(ActionEvent e) {
		labelTime.setText(updateTime());
	    }
	});
	mySecTimer.start();

	labelTime.setFont(headerFont);
	labelTime.setForeground(foregroundColor);
	pane.add(labelTime);
	return pane;
    }

    /**
     * Returns a string with the current time.
     * @return  The time string.
     */    
    private String updateTime() {
	Date now = new Date();
	DateFormat f = DateFormat.getDateTimeInstance(DateFormat.SHORT, DateFormat.SHORT); 
	String d = f.format(now);
        return d;
    }

    /**
     * Changes the panel title.
     * @param title   the name of the main panel.
     */
    public void changeTitlePanel(String title) {
        titleLabel.setText(title);
    }

    /*********** Card Layout from Main Screen ******************/
    
    /**
     * Adds a new panel to the main screen panel card layout.
     * @param newMainScreenPanel    the panel to add in the card layout.
     * @param name            the name that identifies the new panel.
     */    
    public void addToMainScreenCardLayout(JPanel newMainScreenPanel, String name) {
	    mainScreenCard.add(newMainScreenPanel, name );
    }

    /**
     * Restores the initial main screen, removing the previous screen panel.
     * @param screenPanel    the panel to remove from the card layout.
     */    
    public void restoreMainScreen(JPanel screenPanel) {
        mainScreenCard.remove(screenPanel);	
        backToMainScreen();             
    }

    /**
     * Set the correspondig panel to the main panel and shows it.
     * It also changes the title name.
     * @param name  the name of the panel to show.
     */      
    public void setMainScreenPanel( String name) {
	mainScreenPanelLayout.show(mainScreenCard, name);
    }

    /**
     * Set the main panel to menu panel and shows it.
     */        
    public void backToMainScreen() {
        mainScreenPanelLayout.show(mainScreenCard, "INITIAL");
        mainPanel.requestFocusInWindow();
    }    
    
    /**
     * Returns if the application es recording the DVB-T
     * @return      true if it is recording.
     */
    public boolean getIsRecordingDvbt() {
        return isRecordingDvbt;
    }
    
    /**
     * Sets isRecording attribute and the plugin that is running to record
     * @param isRecording   describes if the application is recording or not.
     * @param ep            the ExternalPlugin used to record.
     */    
    public void setIsRecordingDvbt(boolean isRecording, ExternalPlugin ep) {        
        isRecordingDvbt = isRecording;
        if (ep != null) {
            recordingThread = ep;
        }
    }    
    
    /**
     * Returns if the ExternalPlugin that is used to record DVB-T
     * @return      the ExternalPlugin.
     */    
    public ExternalPlugin getRecordingThread() {
        return recordingThread;
    }
    
    /*********** Card Layout from Main Panel ******************/
    
    /**
     * Adds a new panel to the main panel card layout.
     * @param newMainPanel    the panel to add in the card layout.
     * @param name            the name that identifies the new panel.
     */    
    public void addToCardLayout(JPanel newMainPanel, String name) {
	    mainPanel.add(newMainPanel, name );
    }

    /**
     * Set the correspondig panel to the main panel and shows it.
     * It also changes the title name.
     * @param name  the name of the panel to show.
     */      
    public void setMainPanel( String name) {
	mainPanelLayout.show(mainPanel, name);
        changeTitlePanel(name);
    }

    /**
     * Set the main panel to menu panel and shows it.
     */        
    public void backToMenu() {
        mainPanelLayout.show(mainPanel, "MENU");
        changeTitlePanel("Menu");      
	menu.focusedButton.requestFocusInWindow();    
    }
    
    /**
     * Loads the colors and fonts from the config file.
     */      
    private void loadColorFontTheme() {
        ConfigReader config = new ConfigReader();
        titleFont = config.loadFont("title");
        headerFont = config.loadFont("header");
        foregroundColor = config.loadColor("foreground");        
    }
    
    /**
     * Registers the keys to use in the application.
     */    
    private void registerKeys() {
	/* down, tab moves downwards */
	Set<AWTKeyStroke> newForwardKeys = new HashSet<AWTKeyStroke>(2);
      	newForwardKeys.add(AWTKeyStroke.getAWTKeyStroke(KeyEvent.VK_TAB,0));
      	newForwardKeys.add(AWTKeyStroke.getAWTKeyStroke(KeyEvent.VK_DOWN, 0));

      	KeyboardFocusManager.getCurrentKeyboardFocusManager().setDefaultFocusTraversalKeys
				(KeyboardFocusManager.FORWARD_TRAVERSAL_KEYS, newForwardKeys);

	/* up, shift-tab moves upwards */
	Set<AWTKeyStroke> newBackwardKeys = new HashSet<AWTKeyStroke>(2);
      	newBackwardKeys.add(AWTKeyStroke.getAWTKeyStroke(KeyEvent.VK_TAB, KeyEvent.SHIFT_MASK));
      	newBackwardKeys.add(AWTKeyStroke.getAWTKeyStroke(KeyEvent.VK_UP, 0));
      	KeyboardFocusManager.getCurrentKeyboardFocusManager().setDefaultFocusTraversalKeys
				(KeyboardFocusManager.BACKWARD_TRAVERSAL_KEYS, newBackwardKeys);
    }
}