/*
 * MainMenu.java
 *
 * Copyright 2007 Cristina Roura Claver cristina.roura@gmail.com
 *
 * This file is part of LHMC.
 *
 * LHMC is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * LHMC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package lhmc.gui;

import javax.swing.*;          
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.*;

import lhmc.gui.util.*;
import lhmc.gui.plugingui.*;
import lhmc.utils.*;

/**
 * Menu panel.
 * It contains a button panel, each for each plugin available
 * and a image panel, that shows an image representig the plugin.
 *
 * @version 	0.01 Jan 2007
 * @author 	Cristina Roura Claver
 */
public class MainMenu extends JPanel implements FocusListener, ActionListener {

    /* Focused button */
    public CustomButton focusedButton = null;
    
    /* Colors */
    private Color foregroundColor;
    private Color highlightColor;
    
    /* Vectors with each plugin name and its class ubication */
    protected Vector<String> pluginsClassName = new Vector();
    protected Vector<String> pluginsName = new Vector();

    /* Vector of plugins available */
    private Vector<PluginGUI> plugins;  
    
    /* Main frame pointer */
    private LHMCGUI mainFrame = null;

    /* Image representing the plugin option selected */
    private ImageJComponent optionImage = null;     

    /* the configuration reader class */   
    private ConfigReader config = new ConfigReader();
    
    /**
     * Constructs the menu.
     * @param mainFrame LHMC object
     */
    public MainMenu(LHMCGUI mainFrame) {
	this.mainFrame = mainFrame;
                
        loadColorTheme();
           
	setLayout(new BorderLayout());

        mainFrame.changeTitlePanel("Menu");
        
	plugins = new Vector<PluginGUI>();

	add(createComponents(), BorderLayout.CENTER);
    }

    /**
     * Creates all the components of the main menu.
     * Is composed by a buttons list and an image icon.
     * @return      The main menu panel.
     */
    private JPanel createComponents() {
	JPanel pane = new JPanel( );
	pane.setLayout(new BoxLayout(pane, BoxLayout.X_AXIS));
                
	JPanel buttonsPanel = createButtons();
	buttonsPanel.setAlignmentY(Component.CENTER_ALIGNMENT);
	pane.add( buttonsPanel);

	JPanel showPanel = createShowPanel();
	showPanel.setAlignmentY(Component.CENTER_ALIGNMENT);
	pane.add( showPanel);	

	return pane;
    }

    /**
     * Creates the image panel. The image represents the option of the button.
     * @return  The image show panel.
     */    
   private JPanel createShowPanel() {
	JPanel showPane = new JPanel(  new GridLayout(3,0));
	optionImage = new ImageJComponent();

	showPane.add(Box.createRigidArea(new Dimension(0,3))); /* empty space */
	showPane.add( optionImage);
	showPane.add(Box.createRigidArea(new Dimension(0,3))); /* empty space */

	return showPane;
    }

    /**
     * Creates the buttons panel. The buttons number depens on the plugins available.
     * @return      The buttons panel.
     */       
    private JPanel createButtons() {
        
        /* Get the plugins from the configuration file */
        Enumeration enumeration = config.propertyNames();
        while (enumeration.hasMoreElements()) {
            String tempString = (String) enumeration.nextElement();
            if (tempString.startsWith("Class")) {                                        
                pluginsClassName.add(config.getProperty(tempString));
                pluginsName.add(tempString);
            }
        }        
                
        /* order vectors by position */
        orderPluginsVectorsByPosition();
        
        /* Creates the button panel */
	JPanel pane = new JPanel(new GridLayout(pluginsClassName.size()+3, 0));

        /* empty space */
	pane.add(Box.createRigidArea(new Dimension(0,3)));        
                          
        /* buttons creation */
        for (int i = 0 ; i < pluginsClassName.size(); i++) {
            /* for each plguin */
            String pluginClassName = pluginsClassName.elementAt(i);            
            try {
                /* instance a plugin */
                PluginGUI plugin = (PluginGUI) (Class.forName(pluginClassName).newInstance());                                               
                String pluginIcon = searchPluginIcon(pluginsName.elementAt(i));
                plugin.setMediaUrl(searchPluginUrl(pluginsName.elementAt(i)));                
                plugin.setMainFrame(mainFrame);
                plugin.createPanel();
                
                /* Add the plugin to the available plugin vector */
                plugins.add(plugin);                
                                                
                /* adding plugin to the card layout */
                mainFrame.addToCardLayout( plugin, plugin.getTitle() );

                /* creates a new button */
                CustomButton button = new CustomButton();
                button.setText(plugin.getTitle());                
                button.setImageURL(pluginIcon);
                button.addFocusListener(this);
                button.addActionListener(this);		
                if (i == 0) {       /* is the first button */
                    button.requestFocusInWindow(); 
                    focusedButton = button;
                }
                pane.add(button);   /* adds button to the panel */
            } catch (ClassNotFoundException e ) {
                System.out.println("Caught ClassNotFoundException " + e);
            } catch (LinkageError e ) {
                System.out.println("Caught LinkageError " + e );
            } catch (IllegalAccessException e ) {
                System.out.println("Caught IllegalAccessException " + e);
            } catch (InstantiationException e ) {
                System.out.println("Caught InstantiationException " + e);
            } catch (SecurityException e ) {
                System.out.println("Caught SecurityException " + e);
            }                                           
	}
        
        /* creates the exit button */
        CustomButton button = new CustomButton("Exit");        
        button.setImageURL(config.getProperty("IconQuit"));
	button.addFocusListener(this);
	button.addActionListener(this);
        pane.add(button);
        
        /* empty space */
	pane.add(Box.createRigidArea(new Dimension(0,3)));
        
        return pane;
    }
        
    /**
     * Orders the plugins vectors by position.
     */          
    private void orderPluginsVectorsByPosition() {
        int vectorSize = pluginsName.size();
        String[] tempPluginsClassName = new String[vectorSize];
        String[] tempPluginsName = new String[vectorSize];
        int pluginPosition;
        for (int i = 0 ; i < vectorSize; i++) {
            String pluginClassName = pluginsName.elementAt(i);
            pluginPosition = searchPluginPosition(pluginClassName);
            if (pluginPosition != -1 ) {
                tempPluginsClassName[pluginPosition] = pluginsClassName.elementAt(i);
                tempPluginsName[pluginPosition] = pluginsName.elementAt(i);            
            }
        }
        pluginsClassName.removeAllElements();
        pluginsName.removeAllElements();
        for (int i = 0 ; i < vectorSize; i++) {
            pluginsClassName.add(tempPluginsClassName[i]);
            pluginsName.add(tempPluginsName[i]);
        }
    }
    
    /**
     * Searches in the configuration file the icon realated to a plugin.
     * @param pluginClassNameKey    the key name of the plugin class
     * @return      the path of the plugin icon.
     */      
    private String searchPluginIcon(String pluginClassNameKey) {
        String pluginName = pluginClassNameKey.substring(5);
        String pluginIconKey = "Icon" + pluginName;
        String pluginIcon = null;
        pluginIcon = config.getProperty(pluginIconKey);        
        return pluginIcon;        
    }
    
    /**
     * Searches in the configuration file the plugin positon of a plugin.
     * @param pluginClassNameKey    the key name of the plugin class
     * @return      the position number.
     */          
    private int searchPluginPosition(String pluginClassNameKey) {
        String pluginName = pluginClassNameKey.substring(5);
        String pluginPositionKey = "Position" + pluginName;
        int position = -1;
        position = Integer.parseInt(config.getProperty(pluginPositionKey));        
        return position;             
        
    }
    
    /**
     * Searches in the configuration file the plugin class path.
     * @param pluginClassNameKey    the key name of the plugin class
     * @return      the path of the plugin class.
     */       
    private String searchPluginUrl(String pluginClassNameKey) {
        String pluginName = pluginClassNameKey.substring(5);
        String pluginIconKey = "URL" + pluginName;
        String pluginUrl = null;
        pluginUrl = config.getProperty(pluginIconKey);        
        return pluginUrl;        
    }    
    
    /**
     * Returns the plugin GUI object corresponding to the given title.
     * @param title The name of the plugin GUI wanted.
     * @return      The object of the plugin GUI.
     * @see         PluginGUI
     */           
    private PluginGUI getAppByTitle(String title) {
        if (title == null) { return null; }
        /* Searches for the wanted plugin object */
        for (Iterator i = plugins.iterator(); i.hasNext();) {
            PluginGUI app = (PluginGUI)i.next();
            if (title.equals(app.getTitle())) { return app; }
        }
        return null;
    }

    /**
     * Loads the colors from the config file.
     */     
    private void loadColorTheme() {        
        highlightColor = config.loadColor("highlight");        
        foregroundColor = config.loadColor("foreground");        
    }  
    
    /**
     * Invoked when the action button pressed occurs.
     * @param e The action event
     */       
    public void actionPerformed(ActionEvent e) { 
	String action = e.getActionCommand();   
        String text = ((JButton)e.getSource()).getText();   
        if ( text == "Exit" ) {
            System.exit(0);
        } else { /* sets the main panel with the pannel of the option selected */
            PluginGUI app = getAppByTitle(text);
            mainFrame.setMainPanel( text );
            app.focusComponent();
        }
    }

    /**
     * Invoked when a button gains focus.
     * @param e The focus event
     */       
    public void focusGained(FocusEvent e) {
        /* highlights the current button */
        e.getComponent().setForeground(highlightColor);
	
        CustomButton b = (CustomButton)e.getSource();
        
        /* sets the image corresponding to the current button */
        optionImage.setImage( b.getImageURL() );
    }

    /**
     * Invoked when a button loses focus.
     * @param e The focus event
     */           
    public void focusLost(FocusEvent e) {
        /* restore the default color */
        e.getComponent().setForeground(foregroundColor);
    }
    
}