/* ImageUtils.java - niffler.ImageUtils
 *
 * 01.08.06 - out-of-memory warning stuff
 * 19.07.06 - added BufferedImage utilities
 * 27.06.06 - added loadImage...( byte[] ) 
 * 02.03.06 - decide to ignore InterruptedException in load...ViaAWT
 * 18.01.06 - new class, refactored from JImageViewer
 * 02.08.03 - original JImageViewer
 *
 * (C) 2003-2006, Norman Hendrich
 * All rights reserved. See file "niffler/license.txt" for details.
 */

/*
 * Niffler Image Viewer
 * Copyright (c) 2003-2006, Norman Hendrich
 * All rights reserved.
 *
 * Contact:
 *
 * Norman Hendrich
 * c/o University of Hamburg
 * FB Informatik
 * Vogt-Koelln-Str. 30
 * D 22527 Hamburg
 * Germany
 *
 * hendrich@informatik.uni-hamburg.de
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions 
 * are met:
 *
 *   * Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above copyright notice, 
 *     this list of conditions and the following disclaimer in the documentation 
 *     and/or other materials provided with the distribution.
 *   * Neither the name of the University of Hamburg nor the names of its 
 *     contributors may be used to endorse or promote products derived from 
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE 
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER 
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, 
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/**
 * Jun 2007
 * modified by Cristina Roura Claver cristina.roura@gmail.com
 * 
 * CHANGES:
 *      Package name modified.
 *      Working with URL instead of Files: 
 *          loadImage()
 *          loadImageViaToolkit()
 *
 */

package lhmc.gui.thumbnail;


import java.awt.*;
import java.awt.event.*;
import java.awt.geom.*;
import java.awt.image.*;
import java.io.*;
import java.net.URL;
import java.util.*;
import java.net.*;

import jcifs.smb.*;
import javax.imageio.*;
import javax.imageio.*;
import javax.imageio.stream.*;
import javax.imageio.plugins.jpeg.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.tree.*;

public class ImageUtils 
{
  private static boolean debug = false;
  private static boolean outOfMemoryDialogDisabled = false;
  private static boolean outOfMemoryDialogShown = false;


  public static void setDebug( boolean b ) {
    debug = b;
  }


  /**
   * check whether the given file is considered an image file.
   * At the moment, this method returns true for files whose names
   * end with ".gif", ".jpg", ".jpeg", ".png", and ".xbm" after
   * conversion to lowercase.
   *
   * FIXME: this method should allow the user to specify the filename
   * patterns instead of using hardcoded values...
   */
  public static boolean isImageFile( String filename ) {
    String name = filename.toLowerCase();
    if (name.indexOf( ".xvpics" ) >= 0) return false; // skip right here
    // if (name.indexOf( ".niffler/thumbnails" ) >= 0) return false;

    return (    name.endsWith( ".gif" )
             || name.endsWith( ".jpg" ) 
             || name.endsWith( ".jpeg" )
             || name.endsWith( ".png" )
             || name.endsWith( ".xbm" )
             // || name.endsWith( ".pbm" )
             // || name.endsWith( ".pgm" )
             // || name.endsWith( ".ppm" )
             // || name.endsWith( ".pnm" )
           );
  }


  public static String stripDirectoryNames( String pathname ) {
    if (pathname == null) return null;

    // strip off trailing '/', if any
    //
    if (pathname.charAt( pathname.length()-1 ) == '/') {
       pathname = pathname.substring( 0, pathname.length()-1 );
    }

    int pos = pathname.lastIndexOf( '/' );
    if (pos <= 0) return pathname;
    else          return pathname.substring( pos+1, pathname.length() );
  }


  public static ByteArrayOutputStream readStreamIntoBuffer( InputStream input )
  {
    ByteArrayOutputStream baos = null;
    try {
      baos = new ByteArrayOutputStream();

      if (input != null) {
        byte buffer[] = new byte[4096];
        int length =0, count = 0;
        while( (length = input.read( buffer, 0, buffer.length )) >= 0) {
          baos.write( buffer, 0, length );
          count += length;
        }
      }
      baos.close();
      return baos;
    }
    catch( Throwable t ) {
      t.printStackTrace();
    }
    return null;
  }

  
  public static Image loadImage( URL url ) {
    msg( "-#- " );
    msg( "-#- loadImage: trying: '" + url.getPath() + "'" );
 
    /*
    try {
      Class.forName( "javax.imageio.ImageIO" );
      Image tmp = loadImageViaImageIO( pathname );
    }
    catch( Exception e ) {
      return loadImageViaToolkit( pathname );
    }
    */
    // Image tmp = Toolkit.getDefaultToolkit().getImage( pathname );
    Image tmp = loadImageViaToolkit( url, null );
    return tmp;
  }
  
  
  /**
   * load an image (actually BufferedImage) via javax.imageio.ImageIO.read().
   * 
   * Note: while ImageIO.read() often works fine, it can be VERY slow for
   * certain JPEG images. The exact cause for this is unknown to me, 
   * but it seems to happen for JPEG files with certain ICC color profiles.
   * Reading such images can take on the order of ten seconds (or more)
   * on my Athlon 2600+ with JDK 1.4.2 or 1.5.0, even for small images 
   * about 800x600 pixels. Unless this bug is fixed, loadImageViaToolkit()
   * might be the more reliable option.
   *
   * Note: more testing reveals that image loading is pretty fast, but
   * image _rendering_ is slow when using RenderingHints with bilinear
   * interpolation or image quality requested. Rendering is fast without
   * RenderingHints. The 'culprit' images are JPEGs with type BGR and
   * ColorSpace type=5.
   */
  public static Image loadImageViaImageIO( String pathname ) {
    msg( "-#- loading image via ImageIO: '" + pathname + "'" );
    try {
      long t1 = System.currentTimeMillis();

      File tmp = new File( pathname ); 
      BufferedImage im = javax.imageio.ImageIO.read( tmp );
      
      long t2 = System.currentTimeMillis();
      long total = Runtime.getRuntime().totalMemory();
      long  free = Runtime.getRuntime().freeMemory();
      msg( "-#- memory usage: free= " + free + " of total= " + total );
      msg( "-#- loadImage(ImageIO) took " + (t2-t1) + " msec." );
      
      return im;
    }
    catch( OutOfMemoryError oome ) {
      oome.printStackTrace();
      msg( "-E- loadImage(ImageIO): out of memory: '" + pathname + "'" );
      showOutOfMemoryDialog( null );
      return null;
    }
    catch( Exception e ) {
      msg( "-E- Image file invalid: '" + pathname + "'" );
      return null;
    }
  }
  

  /**
   * Try to load an Image from the resource called 'pathname'.
   * If the Image cannot be found, or cannot be decoded, we return null.
   * We use a timeout of at most 10000 msec before returning.
   * <p>
   * The raw image data is loaded via DesignManager.getInputStream()
   * into a ByteArrayOutputStream, and then converted into an Image
   * via java.awt.Toolkit.createImage().
   * <p>
   */
  public static 
  Image loadImageViaToolkit( URL url, ImageObserver observer ) 
  {
    try {
        
        jcifs.Config.registerSmbURLHandler();
        String protocol = url.getProtocol();    
        InputStream inputStream = null;
	if (protocol.equals("file")) {   
            File    file = new File( url.getPath() );   
            inputStream= new BufferedInputStream( new FileInputStream( file ));
        } else if (protocol.equals("smb")) {
            SmbFile smbFile = new SmbFile("smb:/" + url.getPath());   
            inputStream= new BufferedInputStream( new SmbFileInputStream( smbFile ));
        }     
            
        
      long t1 = System.currentTimeMillis();


      if (inputStream == null) {
         throw new FileNotFoundException( "Image not found: " + url.getPath() );
      }

      ByteArrayOutputStream  baos = readStreamIntoBuffer( inputStream );
      inputStream.close();
      byte[] big = baos.toByteArray();
      long    t2 = System.currentTimeMillis();
      msg( "-#- reading into buffer ok, took " + (t2-t1) + " msec." );

      return loadImageViaToolkit( big, observer, url.getPath() );
    }
    catch( OutOfMemoryError oome ) {
      oome.printStackTrace();
      msg( "-E- loadImageViaToolkit: out of memory: '" + url.getPath() + "'" );
      showOutOfMemoryDialog( null );
      return null;
    }
    catch( Exception e ) {
      e.printStackTrace();
      msg( "-E- loadImage: could not load or not found: '" + url.getPath() + "'" );
      return null;
    }
  }




  /**
   * Convert the given raw image data (byte array) into an Image.
   *
   * This method calls Toolkit.createImage() to decode the image,
   * which still works better than the javax.imageio methods on most
   * current Java virtual machines (including JDK 1.4.x, 1.5.x, and classpath).
   * <p>
   * If the cannot be decoded, we return null.
   * We also use a timeout of at most 10000 msec before returning.
   * The optional parameter 'pathname' is only used for (debugging) messages.
   */
  public static 
  Image loadImageViaToolkit( byte[] big, ImageObserver observer, 
                             String pathname ) 
  {
    long t1 = 0; long t2 = 0; long t3 = 0;
    Image image = null;
    boolean bigImageFlag = false;

    try {
      if (big == null) return null;

      t1 = System.currentTimeMillis();
      Toolkit toolkit = Toolkit.getDefaultToolkit();
      image = toolkit.createImage( big );
      t2 = System.currentTimeMillis();

      // manual wait-loop with timeout, MediaTracker does not seem to
      // work for createImage( byte[] )... Also, we want a timeout.
      //
      // NOTE: using a non-null ImageObserver leads to the CRAZY effect
      // that the stupid toolkit won't load animated GIFs. Arrggghhh.
      // 
      toolkit.prepareImage( image, -1, -1, null );  // MUST use null here 

      int max_iterations = 40;
      for( int iter=0; iter < max_iterations; iter++ ) {
        int status = toolkit.checkImage( image, -1, -1, null );

        // check whether we should allow a little bit more time
        // because of large image size
        //
        if (iter == max_iterations-5) { // just once, w&h should be known now
           if (   ((status & ImageObserver.WIDTH) != 0) 
                &&((status & ImageObserver.HEIGHT) != 0)) {

              msg( "-#- loadImage: slow, checking for large image now ..." );
              int w = image.getWidth(null);
              int h = image.getHeight(null);
              if (w*h > 2048*2048) {
                 bigImageFlag = true;
                 max_iterations = 2000; // lots of more time
                 // msg( "-W- loadImage: big image: " + w + "x" + h 
                 //      + " '" + pathname + "'" );
              }
              else if (w*h > 1024*1024) {
                 bigImageFlag = true;
                 max_iterations = 200; // more time
                 // msg( "-#- loadImage: big image, size is " + w + "x" + h 
                 //      + " '" + pathname + "'" );
              }
           }
        }

        // one info message, but only if loading is actually rather slow
        // 
        if (iter == 50) { 
          int w = image.getWidth(null);
          int h = image.getHeight(null);
          msg( "-#- loadImage: " + w + "x" + h + " '" + pathname + "'" );
        }

        if ((status & ImageObserver.ERROR) != 0) {
          msg( "-E- loadImage: error during decoding: '" + pathname + "'" );
          break;
        }
        else if ((status & ImageObserver.ABORT) != 0) { // aborted
          msg( "-E- loadImage: ABORT????" + iter + " iteration(s)" );
          break;
        }
        else if ((status & ImageObserver.ALLBITS) != 0) { // ready
          msg( "-#- loadImage: ready after " + iter + " iteration(s)" );
          break;
        }
        else if ((status & ImageObserver.FRAMEBITS) != 0) { // f*ck anim. GIF
          msg( "-#- loadImage: ignoring GIF animation, "+iter+" iterations(s)");
          break;
        }
        Thread.sleep( 10 ); 

      }
      t3 = System.currentTimeMillis();
    }
    catch( InterruptedException ie ) {
      msg( "-W- loadImage: InterruptedException ignored '" + pathname + "'" );
      if (debug) ie.printStackTrace();
      // if (image != null) image.flush();
    }
    catch( OutOfMemoryError oome ) {
      oome.printStackTrace();
      msg( "-E- loadImage(Toolkit): out of memory: '" + pathname + "'" );
      msg( "-E- image stats: " 
           + Runtime.getRuntime().freeMemory() + " / " 
           + Runtime.getRuntime().totalMemory() + " " 
         );
      oome.printStackTrace();
      showOutOfMemoryDialog( null );
           
      if (image != null) image.flush();
      image = null;
    }
    catch( Exception e ) {
      if (image != null) image.flush();
      e.printStackTrace();
      msg( "-E- loadImage: could not load or not found: '" + pathname + "'" );
    }

    // some status messages (if debug on)
    //
    long total = Runtime.getRuntime().totalMemory();
    long  free = Runtime.getRuntime().freeMemory();
    msg( "-#- memory usage: free= " + free + " of total= " + total );
    msg( "-#- loadImage(AWT) took: total=" + (t3-t1) + " indiv: " 
         + (t2-t1) + " " + (t3-t2) + " msec." );

    return image;
  }


  /** wrapper that creates a BufferedImage with TYPE_INT_ARGB */
  public static BufferedImage createBufferedImageARGB( int w, int h ) {
    return new BufferedImage( w, h, BufferedImage.TYPE_INT_ARGB );
  }


  /** wrapper that creates a BufferedImage with TYPE_INT_RGB */
  public static BufferedImage createBufferedImageRGB( int w, int h ) {
    return new BufferedImage( w, h, BufferedImage.TYPE_INT_RGB );
  }


  public static BufferedImage getBufferedImage( Image image ) {
    return getBufferedImageRGB( image );
  }


  public static BufferedImage getBufferedImageRGB( Image image ) {
    if (image == null) return null;

    if (image instanceof BufferedImage) {
      BufferedImage bi = (BufferedImage) image;
      if (bi.getType() == BufferedImage.TYPE_INT_RGB) return bi;
    }

    // not a BufferedImage, or not the requested type. We create a new
    // BufferedImage, and draw the given image into it.
    //
    long t1 = System.currentTimeMillis();
    int w = image.getWidth( null );
    int h = image.getHeight( null );
    if ((w <= 0) || (h <= 0)) {
      msg( "-E- getBufferedImage: input image has w<=0 and h<=0.");
      msg( "    w= " + w + " h= " + h + " image= " + image );
      return null;
    }

    BufferedImage tmp = new BufferedImage( w,h,BufferedImage.TYPE_INT_RGB );
    Graphics g = tmp.createGraphics();
    g.drawImage( image, 0, 0, null );
    g.dispose();
    long t2 = System.currentTimeMillis();
    ImageUtils.msg( 
      "-#- ImageUtils.getBufferedImageRGB took " + (t2-t1) + " msec.");
    return tmp;
  }


  /**
   * use Component.createImage() to convert the given image into an image
   * that can be rendered quickly on both the JDK/JRE and classpath.
   * This method was suggested by Sven de Marothy on the classpath discuss
   * mailing list as a temporary workaround until the classpath image stuff
   * gets mature.
   */   
  public static Image getCairoBackedImage( Image image, Component comp ) {
    String vendor = System.getProperty( "java.vendor" );
    msg( "-#- getCairoBackedImage: vendor = " + vendor );

    if (image == null) {
      msg( "-E- getCairoBackedImage: image is null, return null!" );
      return image;
    }
    else if (comp == null) {
      msg( "-W- getCairoBackedImage: component should not be null!" );
      return image;
    }
    else if (vendor.indexOf( "GNU" ) >= 0) { // classpath needs workaround...
      int w = image.getWidth( null );
      int h = image.getHeight( null );

      long t1 = System.currentTimeMillis();
      Image cairoBacked = comp.createImage( w, h );
      Graphics        g = cairoBacked.getGraphics();
      g.drawImage( image, 0, 0, null ); 
      g.dispose();
      long t2 = System.currentTimeMillis();

      msg( "-I- getCairoBackedImage: took " + (t2-t1) + " msecs. for "
           + w + "x" + h );

      return cairoBacked;
    }
    else { // the JDK works just fine with Image or BufferedImage
      return image;
    }
  }


  public static BufferedImage rotateImageClockwise( Image image ) {
    long t1 = System.currentTimeMillis();
    try {
      if (image == null) return null;
      int w = image.getWidth( null );
      int h = image.getHeight( null );

      // first, convert the current image to a buffered image, so that
      // we can access the pixels...
      BufferedImage BI = getBufferedImage( image );
      image = BI;

      // next, create a new buffered image and copy the pixels
      BufferedImage BO = 
        // GraphicsConfiguration.createCompatibleImage( h, w );
        new BufferedImage( h,w,BufferedImage.TYPE_INT_RGB);

      for( int i=0; i<w; i++ ) {
        for( int j=0; j<h; j++ ) {
          BO.setRGB( h-1-j,i, BI.getRGB(i,j) );
        }
      }
      long t2 = System.currentTimeMillis();
      msg( "-X- rotateImageClockwise took " + (t2-t1) + " msecs." );
      return BO;
    }
    catch( Throwable t ) {
      msg( "-E- rotateImageClockwise failed: " + t );
      t.printStackTrace();
      return null;
    }
  }


  public static BufferedImage rotateImageCounterClockwise( Image image ) {
    long t1 = System.currentTimeMillis();
    try {
      if (image == null) return null;
      int w = image.getWidth( null );
      int h = image.getHeight( null );

      // first, convert the current image to a buffered image, so that
      // we can access the pixels...
      BufferedImage BI = getBufferedImage( image );
      image = BI;

      // next, create a new buffered image and copy the pixels
      BufferedImage BO = 
        // GraphicsConfiguration.createCompatibleImage( h, w );
        new BufferedImage( h,w,BufferedImage.TYPE_INT_RGB);

      for( int i=0; i<w; i++ ) {
        for( int j=0; j<h; j++ ) {
          BO.setRGB( j,w-1-i, BI.getRGB(i,j) );
        }
      }

      long t2 = System.currentTimeMillis();
      msg( "-X- rotateImageCounterClockwise took " + (t2-t1) + " msecs." );
      return BO;
    }
    catch( Throwable t ) {
      msg( "-E- rotateImageCounterClockwise failed: " + t );
      t.printStackTrace();
      return null;
    }
  }


  /** 
   * rotate the current image via Graphics2D around the given angle
   * (measured in radians).
   * FIXME: This method does not yet use correct offsets... should use
   * the full version from hades.models.imaging.RotateFilter.
   */
  public static BufferedImage 
  rotateImageViaTransformOp( Image image, double angle ) 
  {
    try {
      if (image == null) return null;

      BufferedImage BI = getBufferedImage( image );
      AffineTransform trafo = new AffineTransform();
      trafo.rotate( angle, BI.getWidth()/2, BI.getHeight()/2 );
      AffineTransformOp atop =
        new AffineTransformOp( trafo, AffineTransformOp.TYPE_BILINEAR );
      BufferedImage BO = atop.filter( BI, null );
      return BO;
    }
    catch( Throwable t ) {
      msg( "-E- rotateImageViaTransformOp failed: " + t );
      t.printStackTrace();
      return null;
    }
  }


  /** clip the given integer to the range [0,255] */
  public static int clip255( int value ) {
    if      (value < 0)   return 0;
    else if (value > 255) return 255;
    else                  return value;
  }

  /** clip the given long integer to the range [0,255] */
  public static int clip255( long value ) {
    if      (value < 0)   return 0;
    else if (value > 255) return 255;
    else                  return (int) value;
  }

  /** clip the given double to the range [0.0,1.0] */
  public static double clip01( double value ) {
    if      (value < 0.0) return 0.0;
    else if (value > 1.0) return 1.0;
    else                  return value;
  }

  /** 
   * generate an array of n points that interpolate from (x0,y0) to (x1,y1).
   * Of course, x1 must not be equal to x0.
   */
  public static
  double[] ramp( double x0, double y0, double x1, double y1, int n ) {
    double[] vv = new double[n];
    double   dx = x1-x0;
    double   dy = y1-y0;
    double  factor = dy / ((n+1) * dx);
    for( int i=0; i < n; i++ ) {
      vv[i] = y0 + i * factor;
    }
    return vv;
  }

  /** concatenate two arrays */
  public static double[] concat( double[] a1, double[] a2 ) {
    int N1 = a1.length;
    int N2 = a2.length;
    double[] vv = new double[N1+N2];
    for( int i=0; i < N1; i++ ) {
      vv[i] = a1[i];
    }
    for( int i=0; i < N2; i++ ) {
      vv[i+N1] = a2[i];
    }
    return vv;
  }




  public static final double HUE_UNDEFINED = -1;

  /**
   * convert from RGB to HSV colorspace.
   * The input values RGB are expected to be normalized to range [0,1].
   * Outputs s and v are normalized to range [0,1], while output h (hue)
   * is normalized to the range [0,6]. If s==0, h is HUE_UNDEFINED.
   * Code rewritten after jgt.akpeters.com/papers/SmithLyons96/hsv_rgb.html
   *
   * hue:  red=0 yellow=1 green=2 cyan=3 blue=4 magenta=5
   */
  public static 
  double[] RGB_to_HSV( double r, double g, double b, double[] hsv ) {
    if (hsv == null) hsv = new double[3];

    double x = Math.min( r, Math.min( g, b ));
    double v = Math.max( r, Math.max( g, b ));
 
    if (v == x) {
      hsv[0] = HUE_UNDEFINED;
      hsv[1] = 0.0;
      hsv[2] = v;
      return hsv;
    }

    double f = (r == x) ? (g - b) : ((g == x) ? (b - r) : (r - g));
    double i = (r == x) ? 3 : ((g == x) ? 5 : 1 );

    hsv[0] = i-f/(v-x);
    hsv[1] = (v-x)/v;
    hsv[2] = v;
    return hsv;
  }



  /**
   * convert from (hue,saturation,value) to rgb.
   * Input h is in the range[0,6] or HUE_UNDEFINED, while inputs s and v
   * and the rgb outputs are normalized to the range [0,1].
   */
  public static 
  double[] HSV_to_RGB( double h, double s, double v, double[] rgb ) {
    if (rgb == null) rgb = new double[3];

    if (h == HUE_UNDEFINED) {
      rgb[0] = v; rgb[1] = v; rgb[2] = v;
      return rgb;
    }

    int i = (int) Math.floor( h );
    double f = h - i;
    if (! ((i&1) != 0)) f = 1 - f; // if i is even
    double m = v * (1.0 - s);
    double n = v * (1.0 - s*f);

    switch(i) {
      case 6:
      case 0: rgb[0] = v; rgb[1] = n; rgb[2] = m; break;
      case 1: rgb[0] = n; rgb[1] = v; rgb[2] = m; break;
      case 2: rgb[0] = m; rgb[1] = v; rgb[2] = n; break;
      case 3: rgb[0] = m; rgb[1] = n; rgb[2] = v; break;
      case 4: rgb[0] = n; rgb[1] = m; rgb[2] = v; break;
      case 5: rgb[0] = v; rgb[1] = m; rgb[2] = n; break;
    }
    return rgb;
  }


  /**
   * scale from normalized r,g,b components in range[0,1] to a packed
   * integer pixel ARGB with ranges [0,255] and A=0.
   */
  public static int packRGB( double r, double g, double b ) {
    int ir = clip255( Math.round(r) );
    int ig = clip255( Math.round(g) );
    int ib = clip255( Math.round(b) );

    return (ir << 16) + (ig << 8) + ib;
  }


  /**
   * write the given image to the specifed file in PNG format.
   */
  public static boolean writePNGFile( BufferedImage image, File outfile ) {
    try {
      // no parameters to set here, we just use ImageIO directly.
      // However, we do measure the timing, because this seems to be painfully
      // slow on some VMs - including the JDKs.
      // 
      long t1 = System.currentTimeMillis();
      ImageIO.write( image, "png", outfile );
      long t2 = System.currentTimeMillis();

      ImageUtils.msg( "-I- writePNGFile ok, took " + (t2-t1) + " msec." );
      return true;
    }
    catch (IOException e) {
      e.printStackTrace();
      return false;
    }
  }


  /**
   * write the given image to the specified file in PPM (raw) format.
   */
  public static boolean writePPMFile( BufferedImage image, File outfile ) {
    try {
      // ImageIO doesn't support pnm/ppm out of the box. We save the file
      // by hand...
      int   w = image.getWidth();
      int   h = image.getHeight();
      long t1 = System.currentTimeMillis();
      DataOutputStream dos = 
        new DataOutputStream(  
          new BufferedOutputStream( 
            new FileOutputStream( outfile )));

      dos.writeBytes( "P6\n" );
      dos.writeBytes( "# Niffler " + new java.util.Date() + "\n" );
      dos.writeBytes( "" + w + " " + h + "\n" );
      dos.writeBytes( "255\n" );

      for( int y=0; y < h; y++ ) {
        for( int x=0; x < w; x++ ) {
          int rgb = image.getRGB( x, y );
          byte r = (byte) ((rgb >> 16) & 0xff);
          byte g = (byte) ((rgb >>  8) & 0xff);
          byte b = (byte) ((rgb >>  0) & 0xff);

          dos.write( r );
          dos.write( g );
          dos.write( b );
        }
      }
      dos.flush();
      dos.close();

      long t2 = System.currentTimeMillis();
      ImageUtils.msg( "-I- writePPMFile ok, took " + (t2-t1) + " msec." );
      return true;
    }
    catch (IOException e) {
      e.printStackTrace();
      return false;
    }
  }
  

  /**
   * write the given image to the specifed File in JPEG format at the
   * given quality level.
   * 
   * Returns true on success, false on errors. Use quality=0 for smallest
   * file size, quality=1 for best quality. Values outside this range are
   * interpreted as default quality=0.80.
   * 
   * This method does not check whether the file already exists; the
   * behaviour might be implementation dependent (e.g., my Linux JDK 1.4.2
   * silently renames the original file and then writes the given image).
   * 
   * Note: the JPEG image quality decreases rapidly with the quality level,
   * everything below the default value of 0.80 should only be used when
   * no further processing is planned for that image. Also note that filesize
   * doesn't really decrease, because the algorithm just changes the JPEG
   * quantization tables.
   * 
   * This method is based on the algorithm from the Java Almanac book.
   */
  public static boolean writeJPEGFile( BufferedImage image, File outfile,
                                       double quality ) 
  {
    try {
      long t1 = System.currentTimeMillis();

      if ((quality < 0.0) || (quality > 1.0)) quality = 0.80; // default
      float compressionQuality = (float) quality;

      // find a jpeg writer
      ImageWriter writer = null;
      Iterator iter = ImageIO.getImageWritersByFormatName( "jpg" );
      if (iter.hasNext()) {
        writer = (ImageWriter)iter.next();
        ImageUtils.msg( "-I- writeJPEGFile: using " + writer );
      }
      else {
        ImageUtils.msg( "-E- writeJPEGFile: No JPEG writer found!" );
        return false;
      }

      // prepare output file
      ImageOutputStream ios = ImageIO.createImageOutputStream(outfile);
      writer.setOutput(ios);

      // set the compression quality
// ImageWriteParam iwparam = new ImageUtils().new BugfixImageWriteParam();
ImageWriteParam iwparam = new JPEGImageWriteParam( Locale.getDefault() );
      iwparam.setCompressionMode( ImageWriteParam.MODE_EXPLICIT ) ;
      iwparam.setCompressionQuality( compressionQuality );

      // write the image
      writer.write( null, new IIOImage(image, null, null), iwparam );

      ios.flush();
      writer.dispose();
      ios.close();

      long t2 = System.currentTimeMillis();
      ImageUtils.msg( "-I- writeJPEGFile ok, took " + (t2-t1) + " msec." );

      return true;
    }
    catch (IOException e) {
      e.printStackTrace();
      return false;
    }
  }


  /** 
   * bugfix class as documented in the Java Almanac.
   * See http://javaalmanac.com/egs/javax.imageio/JpegWrite.html
   * 
   * This is required on JDK 1.4 to rescale the compression-quality from
   * the range [0..1] as expected by the javax.imageio framework 
   * to the range [256..0] as implemented in their actual JPEG encoder.
   * Note: I would have expected 255..0, so perhaps the bugfix has a bug
   * itself. Anyway, we use the Almanac code here.
   */
  public class BugfixImageWriteParam extends JPEGImageWriteParam {

    public BugfixImageWriteParam() {
      super(Locale.getDefault());
    }

    // This method accepts quality levels between 0 (lowest) and 1 (highest) i
    // and simply converts it to a range between 0 and 256; this is not 
    // a correct conversion algorithm.
    public void setCompressionQuality(float quality) {
      if (quality < 0.0F || quality > 1.0F) {
        throw new IllegalArgumentException("Quality out-of-bounds!");
      }
      this.compressionQuality = 256 - (quality * 256);
    }
  }


  /**
   * show a dialog window after an out-of-memory error.
   * FIXME: trying to allocate a JOptionPane with HTML payload might not 
   * be a brilliant idea while already out-of-memory. Perhaps we should
   * preallocate the dialog? But this harms startup-time and wastes some
   * memory better used for other things...
   */
  public static void showOutOfMemoryDialog( JFrame parent ) {
    if (outOfMemoryDialogDisabled) {
      return;
    }

    long free  = Runtime.getRuntime().freeMemory() / (1024*1024);
    long total = Runtime.getRuntime().totalMemory() / (1024*1024);

    if (outOfMemoryDialogShown) { // it happened again, shorter concise dialog 
      JOptionPane.showMessageDialog( 
         parent,
         "<html><body>"
         + "The Java virtual machine just ran out of memory again.<br>"
         + "We currently have " + free + " MBytes free, out of<br>"
         + "a total of " + total + " MBytes used.<br>"
         + "</body></html>",
         "Out of memory",
         JOptionPane.ERROR_MESSAGE 
      );
    }
    else { // long, informative first-time warning with -Xmx explanation
      JOptionPane.showMessageDialog( 
         null,
         "<html><body>"
         + "The Java virtual machine just ran out of memory,<br>"
         + "while loading or processing an image.<br>"
         + "We currently have " + free + " MBytes free, out of<br>"
         + "a total of " + total + " MBytes used.<br>"
         + " <br>"
         + "Please increase the memory limit of your Java VM<br>"
         + "and restart the application. Most Java VMs support<br>"
         + "the -Xmx runtime flag to specify the memory limit.<br>"
         + "A value of -Xmx300m is a good starting point for<br>"
         + "typical digicam images up to about 10 MPixels.<br>"
         + "</body></html>",
         "Out of memory",
         JOptionPane.ERROR_MESSAGE 
      );
      outOfMemoryDialogShown = true;
    }
  }



  public static void msg( String s ) {
    if (!debug && s.startsWith( "-#-")) return;
    else System.out.println( s );
  }

} 
