/* SetupManager.java - class niffler.SetupManager
 *
 * A class to manage program environment variables, properties, resources.
 *
 * 23.01.06 - new class (copied from jfig.utils)
 *
 * (C) 2002-2006, Norman Hendrich
 * All rights reserved. See file "niffler/license.txt" for details.
 */

/*
 * Niffler Image Viewer
 * Copyright (c) 2003-2006, Norman Hendrich
 * All rights reserved.
 *
 * Contact:
 *
 * Norman Hendrich
 * c/o University of Hamburg
 * FB Informatik
 * Vogt-Koelln-Str. 30
 * D 22527 Hamburg
 * Germany
 *
 * hendrich@informatik.uni-hamburg.de
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions 
 * are met:
 *
 *   * Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above copyright notice, 
 *     this list of conditions and the following disclaimer in the documentation 
 *     and/or other materials provided with the distribution.
 *   * Neither the name of the University of Hamburg nor the names of its 
 *     contributors may be used to endorse or promote products derived from 
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE 
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER 
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, 
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/**
 * Jun 2007
 * modified by Cristina Roura Claver cristina.roura@gmail.com
 * 
 * CHANGES:
 *      Package name modified.
 * 
 */


package lhmc.gui.thumbnail;

import java.io.*;
import java.util.Enumeration;
import java.util.Properties;



/**
 * find, load, and manager user preferences.
 * <p>
 * SetupManager manages program and user enviroment properties,
 * in four different levels:
 *
 * First, the Java System properties are
 * initialized (which may not be allowed for unsigned Applets).
 * Second, the global properties are loaded via the ClassLoader from
 * a classpath resource, e.g. "/filename".
 * Third, user resources are loaded from a file "$USER/filename" in the
 * user's home directory.
 * Finally, local resources are added from a file "./filename" in the
 * current working directory.
 */
public class SetupManager {

  static SetupManager  _setupManager = null;

  static Properties      properties;                   // all of them merged
  static Properties      globals, userprops, locals;

  static boolean  debug = false;


  static {
    getSetupManager();
  }

 
  public static SetupManager getSetupManager() {
    if (_setupManager == null) _setupManager = new SetupManager();
    return _setupManager;
  }


  private SetupManager() {
    try {
      SecurityManager sm = System.getSecurityManager();
      if (sm != null) sm.checkPropertiesAccess();

      // OK, we are allowed to read the system properties now
      properties = System.getProperties();
    }
    catch( SecurityException se ) {
      // access those system properties allowed for applets...
      properties = new Properties();
      String names[] = {
        "java.version", "java.vendor", "java.vendor.url",
        "java.class.version", "os.name", "os.arch", "or.version",
        "file.separator", "path.separator", "line.separator",
      };
      for( int i=0; i<names.length; i++ ) {
        try {
          properties.put( names[i], System.getProperty( names[i] ));
        }
        catch( SecurityException sse ) {
          ; // empty
        }
      }
    }
    catch( Throwable e ) {
      // we shouldn't arrive here, but as a last resort...
      properties = new Properties();
    }
  }


  /** 
   * returns the filename of the directory that Niffler uses as its base
   * base directory.
   * Unless overridden by an explictly set "niffler.home" property,
   * the default location is "[user.home]/.niffler".
   */
  public static String getNifflerDirectory() {
    try {
      // first check the 'niffler.home' property and use it if set
      String s = System.getProperty( "niffler.home" );
      if (s != null) return s;

      // fall back to <user.home>/.niffler
      String t = System.getProperty( "user.home" );
      return t + File.separatorChar + ".niffler";
    }
    catch( Exception e ) {
      // we arrive here when running as an unsigned applet, or when the
      // directory specified by an explicitly-set niffler-home property
      // doesn't exist... We try .niffler in the current directory.
      // 
      ImageUtils.msg( "-E- getNifflerDirectory: " + e );
      return "./.niffler";
    }
  }



  public static Properties getProperties() {
    return properties;
  }



  public static String getProperty( String key, String defaultvalue ) {
    return properties.getProperty( key, defaultvalue );
  }

  public static String getProperty( String key ) {
    return properties.getProperty( key ); // may be null!
  }


  public static int getInteger( String key, int defaultvalue ) {
// System.out.println( "-#- getInteger: '" + key + "' " + defaultvalue );
    String s = getProperty( key );
// System.out.println( "value='" + s  + "'" );
    if (s == null) return defaultvalue;
    else {
      try {
        int v = Integer.parseInt( s.trim() );
// System.out.println( "parsed=" + v );
        return v; 
      }
      catch( NumberFormatException e ) {
if (debug) e.printStackTrace();
        return defaultvalue;
      }
    }
  }


  public static boolean getBoolean( String key, boolean defaultvalue ) {
    String s = getProperty( key );
    if (s == null) return defaultvalue;
    else {
      if      (s.trim().toLowerCase().equals( "true" ))  return true;
      else if (s.trim().toLowerCase().equals( "false" )) return false;
      else                                        return defaultvalue;
    }
  }

  public static double getDouble( String key, double defaultvalue ) {
    String s = getProperty( key );
    try {
      return Double.valueOf( s.trim() ).doubleValue();
    }
    catch( Exception e ) {
      return defaultvalue;
    }
  }

  
  public static java.awt.Color 
  getColor( String key, java.awt.Color defaultcolor ) {
    String s = getProperty( key );
    if (s == null) return defaultcolor;
    else {
      try {
        java.awt.Color c = new java.awt.Color( Integer.decode(s).intValue() );
        return c;
      }
      catch( Exception e ) {
        ImageUtils.msg( "-E- Invalid Color '" + s + "' for key: " + key );
        return defaultcolor;
      }
    }
  }
 

  public static void setProperty( String key, String value ) {
    if (key != null && value != null) {
      properties.put( key, value );
    }
  }

  public static void setBoolean( String key, boolean value ) {
    setProperty( key, (value ? "true" : "false") );
  }


  public static void loadAllProperties( String filename ) {
    loadGlobalProperties( filename );
    loadUserProperties( filename );
    loadLocalProperties( filename );
  }


  /**
   * load "global" properties from the resource named "filename".
   * To get a valid Java classpath resource name, this method automatically
   * prepends a "/" to filename, of none is specified.
   */
  public static void loadGlobalProperties( String filename ) {
    if (debug) ImageUtils.msg( "-I- loading global props: " + filename );

    getSetupManager(); // create on demand
    globals = new Properties();
    InputStream IS = null;
    try { 
      if (!filename.startsWith( "/" )) filename = "/" + filename;
      IS = SetupManager.class.getResourceAsStream( filename );
      globals.load( IS );
    }
    catch( Exception e ) {
      ImageUtils.msg(
        "-W- Couldn't load global properties from '" + filename + "'" );
e.printStackTrace();
    }
    merge( properties, globals );
  }


  public static void loadUserProperties( String filename ) {
    getSetupManager(); // create on demand

    userprops = new Properties();
    FileInputStream FS = null;
    String fname = null;
    try {
      fname =   properties.getProperty( "user.home" ) 
              + properties.getProperty( "file.separator" )
              + filename;
      if (debug) ImageUtils.msg( "-I- loading user props: " + fname );

      FS = new FileInputStream( fname );
      userprops.load( FS );
      FS.close();
    }
    catch( Exception e ) {
      ImageUtils.msg( 
        "-W- Couldn't load user properties from '" + fname + "'" );
    }
    merge( properties, userprops );
  }


  public static void loadLocalProperties( String filename ) {
    getSetupManager(); // create on demand

    FileInputStream FS = null;
    String fname = null;
    try {
      fname =   properties.getProperty( "user.dir" ) 
              + properties.getProperty( "file.separator" )
              + filename;
      if (debug) ImageUtils.msg( "-I- loading local props: " + fname );
      FS = new FileInputStream( fname );
      loadLocalProperties( FS );
    }
    catch( Exception e ) {
      ImageUtils.msg( "-W- Couldn't load local properties '" + fname + "'" );
    }
  }



  public static void loadLocalProperties( InputStream IS ) {
    locals = new Properties();
    try {
      locals.load( IS );
      IS.close();
    }
    catch( Exception e ) {
      ImageUtils.msg( "-W- Couldn't load local properties '" + IS + "'" );
    }
    merge( properties, locals );
  }


  public static void loadURLProperties( String urlname ) {
    getSetupManager();
    locals = new Properties();
    try {
      java.net.URL url = new java.net.URL( urlname );
      url.openConnection();
      InputStream IS = url.openStream();
      locals.load( IS );
      IS.close();
    }
    catch( Exception e ) {
      ImageUtils.msg( "-E- in LoadURLProperties '" + urlname + "' " + e );
    }
    merge( properties, locals );
  }



  public static void save( OutputStream OS, String label ) {
    try {
      properties.store( OS, label );
    }
    catch( Exception e ) {
      ImageUtils.msg( "-E- Failed to save the properties: " + e );
      e.printStackTrace();      
    }
  }


  public static void saveUserProperties( String label, String filename ) {
    OutputStream FS = null;
    String fname = null;
    try {
/* relative to the user's home directory?!
      fname =   getProperty( "user.home" ) 
              + getProperty( "file.separator" )
              + filename;
*/
      fname = filename;

      FS = new FileOutputStream( fname );
      Properties p = new Properties();
      merge( p, userprops );
      merge( p, locals );

      p.store( FS, label );

      FS.flush();
      FS.close();
      ImageUtils.msg( "-I- Saved setup to file '" + fname + "'" );
    }
    catch( Exception e ) {
      if (debug) {
        ImageUtils.msg( "-W- loadLocalProperites: " + e );
        ImageUtils.msg( "-W- Couldn't load local properties '" + fname + "'" );
      }
    }
  }


  public static void saveLocalProperties( String label, String filename ) {
    OutputStream FS = null;
    String fname = null;
    try {
      fname =   getProperty( "user.dir" ) 
              + getProperty( "file.separator" )
              + filename;
      FS = new FileOutputStream( fname );
      Properties p = new Properties();
      merge( p, userprops );
      merge( p, locals );
      p.store( FS, label );
      FS.flush();
      FS.close();
      ImageUtils.msg( "-I- Saved setup to file '" + fname + "'" );
    }
    catch( Exception e ) {
      ImageUtils.msg( "-E- saveLocalProperites: " + e );
      ImageUtils.msg( "-E- Couldn't save local properties '" + fname + "'" );
    }
  }



  public static void merge( Properties dest, Properties src ) {
    String pname = null;

    for( Enumeration e = src.propertyNames(); e.hasMoreElements(); ) {
      pname = (String) e.nextElement();
      dest.put( pname, src.getProperty( pname ));
    }
  }

 
  public static void list( PrintStream ps ) {
    ps.println( getSetupManager().toString() + "[" );
    String pname;
    for( Enumeration e = properties.propertyNames(); 
         e.hasMoreElements(); ) 
    {
      pname = (String) e.nextElement();
      ps.println( pname + " " + getProperty( pname ));
    }
    ps.println( "]" );
  } 



  public static void main( String argv[] ) {
    ImageUtils.msg( "SetupManager selftest..." );
    SetupManager setupManager = getSetupManager();
    SetupManager.loadAllProperties( "jfig.cnf" );
    SetupManager.list( System.out );

    ImageUtils.msg( "Saving user Properties..." );
    SetupManager.saveUserProperties( "jfig setup variables", "hugo.conf" );
  }

}

