/* ThumbnailsCreator.java - niffler.ThumbnailsCreator
 *
 * 19.01.06 - new class (refactored from ThumbnailsManager)
 *
 * (C) 2006, Norman Hendrich
 * All rights reserved. See file "niffler/license.txt" for details.
 */

/*
 * Niffler Image Viewer
 * Copyright (c) 2003-2006, Norman Hendrich
 * All rights reserved.
 *
 * Contact:
 *
 * Norman Hendrich
 * c/o University of Hamburg
 * FB Informatik
 * Vogt-Koelln-Str. 30
 * D 22527 Hamburg
 * Germany
 *
 * hendrich@informatik.uni-hamburg.de
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions 
 * are met:
 *
 *   * Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above copyright notice, 
 *     this list of conditions and the following disclaimer in the documentation 
 *     and/or other materials provided with the distribution.
 *   * Neither the name of the University of Hamburg nor the names of its 
 *     contributors may be used to endorse or promote products derived from 
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE 
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER 
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, 
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/**
 * Jun 2007
 * modified by Cristina Roura Claver cristina.roura@gmail.com
 * 
 * CHANGES:
 *      Package name modified.
 * 
 */

package lhmc.gui.thumbnail;

import java.awt.*;
import java.awt.image.*;
import java.io.*;
import java.util.Vector;

import javax.imageio.*;
import javax.swing.*;


/**
 * helper class to create image thumbnails in a background worker thread.
 */
public class ThumbnailsCreator extends Thread {
  private ThumbnailsManager manager;
  private int DEFAULT_SLEEP_MILLIS = 1000;

  private Vector   filenameQueue;         // String
  private Vector   thumbnailsQueue;       // BufferedImage[]
  private Vector   thumbnailsIndexQueue;  // Integer 
  private Vector   repaintQueue;          // JComponent
  private String   lock = "lock";
  private boolean  finished = false;


  /**
   * create a new background worker thread to create thumbnail images
   * for the given thumbnails manager.
   * Don't forget to call start() on the newly create ThumbnailsCreator
   * to actually start the thread. You can later call setFinished(true)
   * to stop the thread at the next opportune moment.
   */
  public ThumbnailsCreator( ThumbnailsManager _manager ) {
    manager = _manager;
    filenameQueue        = new Vector();
    thumbnailsQueue      = new Vector();
    thumbnailsIndexQueue = new Vector();
    repaintQueue         = new Vector();

    setPriority( Thread.MIN_PRIORITY );
    setDaemon( true );
  }


  /**
   * add a new image to the worker's list of orders.
   * 
   * The given image file will be converted to a image thumbnail,
   * and then be stored into thumbnailsArray[thumbnailsIndex].
   * Finally, a repaint is requested for component target (if non-null).
   * <p>
   * A new order will be processed before existing orders, so that the
   * currently visible thumbnails will be created first, when the user
   * rapidly places multiple orders.
   */
  public void addThumbnailOrder( String imageFilename,
                                 BufferedImage[] thumbnailsArray,
                                 int             thumbnailsIndex,
                                 JComponent      target ) 
  {
    synchronized (lock) {
      filenameQueue.insertElementAt( imageFilename, 0 );
      thumbnailsQueue.insertElementAt( thumbnailsArray, 0 );
      thumbnailsIndexQueue.insertElementAt( new Integer(thumbnailsIndex), 0 );
      repaintQueue.insertElementAt( target, 0 );
    }
    ImageUtils.msg( "-#- added order: " 
                    + imageFilename + " " 
                    + filenameQueue.size() );
    // this.interrupt();
  }


  /**
   * tell the worker thread to finish at the next opportune moment,
   * but wait until an ongoing thumbnail has been created.
   */
  public void setFinished( boolean b ) {
    finished = b;
    // this.interrupt();
  }


  /**
   * endless loop to process the orders posted by the clients.
   * Once started, the thread will awake every 100 milliseconds
   * (DEFAULT_SLEEP_MILLS) and check whether new orders have been placed.
   * The loop exists after setFinished(true) has been called.
   */ 
  public void run() {
    while (!finished) {
      try {

        String        filename = null;
        BufferedImage[] thumbs = null;
        Integer          index = null;
        JComponent      target = null;

        synchronized(lock) {
          if (filenameQueue.size() > 0) {
            filename = (String) filenameQueue.elementAt(0);
            thumbs   = (BufferedImage[]) thumbnailsQueue.elementAt(0);
            index    = (Integer) thumbnailsIndexQueue.elementAt(0);
            target   = (JComponent) repaintQueue.elementAt(0);

            filenameQueue.removeElementAt(0);
            thumbnailsQueue.removeElementAt(0);
            thumbnailsIndexQueue.removeElementAt(0);
            repaintQueue.removeElementAt(0);
          }
        }

        if (filename != null) {
          BufferedImage tmp = manager.checkCreateThumbnail( filename, null );
          if (thumbs != null) thumbs[index.intValue()] = tmp;
          if (target != null) target.repaint();
        }
        else { // don't sleep when busy
          Thread.sleep( DEFAULT_SLEEP_MILLIS );
        }
      }
      catch( InterruptedException ie ) {
        ImageUtils.msg( "-#- ThumbnailsCreator: interrupted!" );
      }
      catch( OutOfMemoryError oome ) {
        ImageUtils.msg( "-E- ThumbnailsCreator: got an OutOfMemoryError.\n"
          + "You may have to restart the application. Please consider\n"
          + "using the -Xmx option (or similar) to increase the memory\n"
          + "limit for the Java virtual machine, for example\n"
          + "java -Xmx512m niffler.Niffler <args>\n" );
         oome.printStackTrace();
      }
      catch( Throwable e ) {
        ImageUtils.msg( "-#- ThumbnailsCreator: " + e );
        e.printStackTrace();
      } 
    } // while 
    ImageUtils.msg( "-#- ThumbnailsCreator: finished." );
  }

} // ThumbnailsCreator
