/* ThumbnailManager.java - niffler.ThumbnailManager
 *
 * 21.06.06 - use ExifManager to load preview thumbnail, when available
 * 19.01.06 - more finetuning, made worker an external class
 * 18.01.06 - finetuning
 * 17.01.06 - first working version
 * 29.09.05 - some code
 *
 * (C) 2005-2006, Norman Hendrich
 * All rights reserved. See file "niffler/license.txt" for details.
 */

/*
 * Niffler Image Viewer
 * Copyright (c) 2003-2006, Norman Hendrich
 * All rights reserved.
 *
 * Contact:
 *
 * Norman Hendrich
 * c/o University of Hamburg
 * FB Informatik
 * Vogt-Koelln-Str. 30
 * D 22527 Hamburg
 * Germany
 *
 * hendrich@informatik.uni-hamburg.de
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions 
 * are met:
 *
 *   * Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above copyright notice, 
 *     this list of conditions and the following disclaimer in the documentation 
 *     and/or other materials provided with the distribution.
 *   * Neither the name of the University of Hamburg nor the names of its 
 *     contributors may be used to endorse or promote products derived from 
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE 
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER 
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, 
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/**
 * Jun 2007
 * modified by Cristina Roura Claver cristina.roura@gmail.com
 * 
 * CHANGES:
 *      Package name modified.
 *	default size to 200.
 *	set the size in constructor to 200.
 *	Don't want to use exif, set the boolean to false in: createThumbnailImage( cooked, originalImage, false ).
 *	Only want to get the thumbnail, not the filling part so, 
 *	in method BufferedImage paintScaledThumbnail( BufferedImage image ):
 *	    commented the filling part and draw without the offset.
 *      int method createThumbnailImage(), erased exif part.
 *      Modified to user URL instead of File:     
 *          checkCreateThumbnail() 
 *          updateThumbnail()
 *          createThumbnailImage()
 *      
 */

package lhmc.gui.thumbnail;

import java.awt.*;
import java.awt.image.*;
import java.io.*;
import java.lang.ref.*;
import java.util.*;
import javax.imageio.*;
import javax.swing.*;
import java.net.*;

import jcifs.smb.*;

import lhmc.utils.*;

/**
 * creates and manages image thumbnails.
 */
public class ThumbnailsManager {
  private boolean debug = false; 
  private int cache_limit = 2000; //2000 default

  private int           size    = 200;                // default size
  private Color backgroundColor = Color.LIGHT_GRAY;   // default background
 
  private String extension = ".thumb";
  private String thumbnailsBaseDirectoryName;
  private Hashtable cache;

  private BackgroundLoader backgroundLoader = null;
  private int n_created = 0;


  /**
   * creates a new thumbnails manager.
   */
  public ThumbnailsManager() {
    cache = new Hashtable();
    thumbnailsBaseDirectoryName = getThumbnailsBaseDirectoryName();

    size            = SetupManager.getInteger( "niffler.thumbnailSize", 200 );
    backgroundColor = SetupManager.getColor( "niffler.thumbnailBackground",
                                             Color.LIGHT_GRAY );
    cache_limit = SetupManager.getInteger( "niffler.thumbnailCacheSize", 2000 ); //2000 default

    backgroundLoader = new BackgroundLoader();
    backgroundLoader.start();

  }  


  /**
   * clear the thumbnails cache, no questions asked. 
   * Note that any "ERROR" entries will also be cleared.
   */
  public void clearCache() {
    ImageUtils.msg( "-X- ThumbnailsManager.clearCache: " + cache.size() );
    Hashtable tmp = cache;
    cache = new Hashtable(); 

    for( Enumeration e = tmp.keys(); e.hasMoreElements(); ) {
      Object o = tmp.get( e.nextElement() );
      if (o instanceof Image) {
        ((Image) o).flush();
      }
    }

    // tell the JVM this is a good time to clean up...
    System.gc();

    ImageUtils.msg( "-X- ThumnailsManager: free memory= " 
                    + Runtime.getRuntime().freeMemory() );
  }


  public void checkCacheSizeLimit() {
    if (cache.size() > cache_limit) {
      ImageUtils.msg( "-X- checkCacheSizeLimit: cache size is " 
                      + cache.size() + ", clearing now..." );
      clearCache();
    }
  }


  /**
   * request a thumbnail image for the thumnails cache.
   * If the thumbnail is cached, it is returned right away.
   * Otherwise, this method returns null (quickly), and ask the background
   * thumbnails loading thread to load (and possibly first create) the
   * thumbnail image. Once the background thread has loaded a thumbnail,
   * the 'target' JComponent (if non-null) is repainted.
   * 
   * Unfortunately, using SoftReference objects for the thumbnails cache
   * doesn't work too well on current JDKs, because the cache items are
   * deleted very frequently.
   * Therefore, hard references are used right now; but the whole cache
   * might be cleared when an OutOfMemoryError occurs.
   */
  public BufferedImage getCachedThumbnailOrNull( String imagePathName,
                                                 JComponent target ) 
  {
    ImageUtils.msg( "-#- getCachedThumbnailOrNull: '" + imagePathName + "'" );

    Object o = cache.get( imagePathName );

    if ("ERROR".equals(o)) { // we already tried this before, but failed
      ImageUtils.msg( "-E- getCachedThumbnailOrNull: error '" 
                      + imagePathName + "'" );
      return null;
    }

    if ("BUSY".equals(o)) { // we already ordered this, but not ready yet
      ImageUtils.msg( "-#- getCachedThumbnailOrNull: busy '" 
                      + imagePathName + "'" );
      return null;
    }

    if ((o != null) && (o instanceof BufferedImage)) {
      ImageUtils.msg( "-#- getCachedThumbnailOrNull: cached '" 
                      + imagePathName + "'" + o.hashCode() );

      return (BufferedImage) o;
    }

    if ("ONE_FAILURE".equals(o)) { // this failed once, we try again
      ImageUtils.msg( "-#- getCachedThumbnailOrNull: second attempt '" 
                      + imagePathName + "'" );
      backgroundLoader.addThumbnailOrder( imagePathName, target );
      return null;
    }

    // when we arrive here, the thumbnail is not cached yet. 
    // We order its production and return null for now.

    ImageUtils.msg( "-#- getCachedThumbnailOrNull: ordering '" 
                      + imagePathName + "'" );

    backgroundLoader.addThumbnailOrder( imagePathName, target );
    return null;
  }


  /** 
   * calculates the file (name) used for the thumbnail for the given
   * original image name.
   * This method handles canonical or non-canonical file names, 
   * and also the special (degenerate) case of the original image already 
   * inside the thumbnails directory tree.
   */
  public File getThumbnailFilename( String imagePathName ) throws IOException
  {
      
      
      // we want the canonical form to find cached thumbnails even when
      // started from different working directories...
      // 
      File    raw = new File( imagePathName );
      File cooked = raw.getCanonicalFile();
      
      // long  t0 = cooked.lastModified();

      String tndir = null;
      String  name = null;
 
      if (isInsideThumbnailsDirectory( cooked )) {
        tndir = cooked.getParent(); 
        name  = cooked.getName(); // no extra extension

        ImageUtils.msg( "-#- TM.getThumbnail: already a thumbnail: " 
                        + tndir + " " + name );
      }
      else {
        tndir = getThumbnailsDirectoryName( cooked.getParent() );
        name  = cooked.getName() + extension;
      }

      File tnFile = new File( tndir, name );
      return tnFile;
  }


  /**
   * return the cached thumbnail image for the given source image,
   * or null if no thumbnail exists.
   * <p>
   * This method does NOT trigger the background creation of thumbnails
   * that don't exist. It also does NOT check whether an existing thumbnail
   * is up to date. Use checkCreateThumbnail instead.
   * <p>
   * This method handles the special case of a thumbnail being requested 
   * for a thumbnail image by returning the thumbnail... the extra check
   * seems cheap when compared to the complications arising otherwise.
   */
  public BufferedImage getThumbnail( String imagePathName ) {
    try {
      File tnFile = getThumbnailFilename( imagePathName );
      if (tnFile.exists()) {
        // no check whether thumbnail is up to date...
        BufferedImage bi = readThumbnail( tnFile );

        // cache.put( imagePathName, new SoftReference( bi ) ); 
        checkCacheSizeLimit();
        cache.put( imagePathName, bi );
        return bi;
      }
      else {
        // checkCacheSizeLimit(); // one extra ERROR entry does not hurt
        cache.put( imagePathName, "ERROR" );
        return null;
      }
    }
    catch( Throwable t ) {
      t.printStackTrace();
      return null;
    }
  }


  /** 
   * return a thumbnail image for the specified original image.
   * <p>
   * This first calculates the thumbnail directory name corresponding
   * to the source image directory, and creates the thumbnail directory
   * if necessary.
   * <p>
   * Next, we check whether the thumbnail exists. If yes, we check whether
   * the thumbnail is up-to-date (newer than the original image).
   * If so, we read the thumbnail image and return it.
   * Otherwise, we create the thumbnail image and store it into the
   * thumbnails directory, before returning the thumbnail image.
   * 
   * FIXME: this should probably also check the file modification times
   * before deciding whether to use or (re-) create a thumbnail.
   */
  public BufferedImage checkCreateThumbnail( String imagePathName, 
                                             BufferedImage originalImage )
  {
 
    try {
      // we want the canonical form to find cached thumbnails even when
      // started from different working directories...
      // 

        jcifs.Config.registerSmbURLHandler();
        URL    url = new URL( imagePathName );
        String protocol = url.getProtocol();
        long     t0 = 0;
        String tndir = null;
        String  name = null;        
        String filePath = null;
	if (protocol.equals("file")) {   
            File    raw = new File( url.getPath() );
            File cooked = raw.getCanonicalFile();
            t0 = cooked.lastModified();   
            tndir= getThumbnailsDirectoryName( cooked.getParent() );
            name = cooked.getName() + extension;  
            filePath = cooked.getPath();
        } else if (protocol.equals("smb")) {
            SmbFile raw = new SmbFile("smb:/" + url.getPath());    
            SmbFile cooked = new SmbFile(raw.getCanonicalPath());            
            t0 = cooked.lastModified();
            tndir= getThumbnailsDirectoryName( cooked.getParent() );
            name = cooked.getName() + extension;              
            filePath = cooked.getPath();
        }                   
                
      File tnFile = new File( tndir, name );

if (debug) {
System.out.println( "-#- gTFI '" + imagePathName + "'" );
System.out.println( "   tndir '" + tndir + "'" );
System.out.println( "  tnfile '" + tnFile.getCanonicalPath() + "'" );
System.out.println( " up-to-date: " + (tnFile.lastModified() > t0)
                       + " '" + imagePathName + "'" );
}


      if (tnFile.exists() && (tnFile.lastModified() > t0)) {
        BufferedImage bi = readThumbnail( tnFile );
        checkCacheSizeLimit();
        cache.put( imagePathName, bi );
        return bi;
      }
      else {
        checkCreateThumbnailsDirectory( new File(tndir) );

        // DON'T allow using the exif thumbnail, if any
        BufferedImage tn = createThumbnailImage( url, originalImage, false );

        if (tn != null) {
          writeThumbnail( tn, tnFile.getCanonicalPath() );
          n_created++;

          ImageUtils.msg( "-I- created thumbnail (" 
                          + n_created 
                          + "/" + backgroundLoader.countPendingOrders()
                          + "/" + cache.size() + "): '" 
                          + name );
                          // + imagePathName );

          checkCacheSizeLimit();
          cache.put( imagePathName, tn );
        }
        else { // failed
          if ("ONE_FAILURE".equals( cache.get( imagePathName ))) {
               cache.put( imagePathName, "ERROR" );
          }
          else cache.put( imagePathName, "ONE_FAILURE" );
        }

        return tn;
      }
    }
    catch( Throwable t ) {
      t.printStackTrace();
      return null;
    }
  }


  /**
   * delete the thumbnail image file for the given source image file.
   */
  public void deleteThumbnail( String imagePathName ) {
    try {
      File raw    = new File( imagePathName );
      File cooked = raw.getCanonicalFile();

   
      String tndir = getThumbnailsDirectoryName( cooked.getParent() );
      String  name = cooked.getName() + extension;

      File tnFile = new File( tndir, name );

      if (tnFile.exists()) {
        tnFile.delete();
        cache.put( imagePathName, "ERROR" );
      }
      ImageUtils.msg( "-I- delete thumbnail for '" + imagePathName + "'" );
    }
    catch( Throwable t ) {
      t.printStackTrace();
    }
  }


  /**
   * delete the thumbnail directory for the given source image directory.
   * If an image file (instead of directory) is specified, we delete the
   * thumbnails for the corresponding directory.
   * This also unconditionally deletes everything in that directory.
   */
  public void deleteThumbnailDirectory( String imageDirectoryName ) {
    try {
      File raw    = new File( imageDirectoryName );

      if (!raw.exists()) return;
      if (!raw.isDirectory()) {
        raw = new File( raw.getParent() );
        if (!raw.exists()) return;
      }

      File cooked = raw.getCanonicalFile();
      File tndir =  new File( getThumbnailsDirectoryName( cooked.getPath() ));

      String[] names = tndir.list();
      for( int i=0; i < names.length; i++ ) {
        new File( tndir, names[i] ).delete();
      }
      tndir.delete();
    }
    catch( Throwable t ) {
      t.printStackTrace();
    }
  }


  /**
   * Intended to (re-) create a thumbnail from the given source image,
   * even if the thumbnail already exists.
   */
  public void updateThumbnail( String imagePathName, Image image ) {
    try {
      ImageUtils.msg( "-X- updateThumbnail: '" + imagePathName + "'" );

      if (image == null) {
        jcifs.Config.registerSmbURLHandler();
        URL    url = new URL( imagePathName );
                    
        image = ImageUtils.loadImage( url );
      }
      BufferedImage bi = ImageUtils.getBufferedImage( image );
      BufferedImage thumb = paintScaledThumbnail( bi ); 
      File tnfile = getThumbnailFilename( imagePathName );
      String tncp = tnfile.getCanonicalPath();

      writeThumbnail( thumb, tncp );
      cache.put( imagePathName, thumb );

      ImageUtils.msg( "-X- re-created the thumbnail '" + tncp + "'" );
      ImageUtils.msg( "    image is " + thumb );
    }
    catch( Throwable t ) {
      t.printStackTrace();
    }
  }


  /**
   * return a BufferedImage for the given thumbnail file.
   * This method currently uses ImageIO.read.
   */
  public BufferedImage readThumbnail( File file ) {
    try { 
      BufferedImage thumbnail = ImageIO.read( file );
      return thumbnail;
    }
    catch( Exception e ) {
      ImageUtils.msg( "-W- readThumbnail failed for file '" + file + "'" );
      e.printStackTrace();
      return null;
    }
  }


  /**
   * check whether the given directory exists; create it if necessary.
   */
  public void checkCreateThumbnailsDirectory( File dir ) {
    if (!dir.exists()) dir.mkdirs();
  }


  /**
   * create a special "directory thumbnail" image for the given directory.
   * The resulting BufferedImage includes the fixed text "directory" 
   * and the "name" of the given directory. 
   */
  public BufferedImage createDirnameThumbnail( String dir ) {
    ConfigReader config = new ConfigReader();
    String name = dir;

    BufferedImage tmp = new BufferedImage( size, size,
                                           BufferedImage.TYPE_INT_RGB );
    Graphics g = tmp.getGraphics();
    g.setColor( backgroundColor );
    g.fillRect( 0, 0, size+1, size+1 );

    RenderingHints hints = new RenderingHints(null);
    hints.put(RenderingHints.KEY_RENDERING,
              RenderingHints.VALUE_RENDER_QUALITY);
    hints.put(RenderingHints.KEY_INTERPOLATION,
              RenderingHints.VALUE_INTERPOLATION_BILINEAR);

    // we need extra precaution here, because GCJ/Classpath (as of 2006.02)
    // still crash on this...
    try {
      if (g instanceof Graphics2D) {
        ((Graphics2D)g).setRenderingHints( hints );
      }
    }
    catch( Throwable t ) {
      t.printStackTrace();
    }

    // g.setColor( Color.white );
    g.setColor( Color.BLACK );
    g.drawString( "directory", 11, size/4 );
    g.drawString( name, 11, 3*size/4 );

    return tmp;
  }


  /**
   * create a special "error thumbnail" for the given file.
   */
  public BufferedImage createErrorThumbnail( String path ) {
    String name = path;

    BufferedImage tmp = new BufferedImage( size, size,
                                           BufferedImage.TYPE_INT_RGB );
    Graphics g = tmp.getGraphics();
    g.setColor( backgroundColor );
    g.fillRect( 0, 0, size+1, size+1 );

    g.setColor( Color.white );
    g.drawString( "not an image", 11, size/4 );
    g.drawString( name, 11, 3*size/4 );

    return tmp;
  }

 
  /**
   * create a thumbnail for the given source image.
   * If the image input parameter is non-null, a thumbnail-sized image
   * is created from that image right away. Otherwise, the source image
   * is read from the given source File first, and then rescaled to
   * thumbnail size.
   */
  public BufferedImage createThumbnailImage( URL imageURL, 
                                             BufferedImage image,
                                             boolean allowExifThumbnail
                                           )                                            
  {
      
    ImageUtils.msg( "-#- createThumbnailImage: '" + imageURL.getPath() + "'" );
    if (imageURL == null) return createErrorThumbnail( imageURL.getPath() );
/*
 if (!imageFile.exists()) return createErrorThumbnail( imageURL );
    if (imageFile.isDirectory()) {
      return createDirnameThumbnail( imageURL );
    }
*/
    // DON'T try to read the preview thumbnail from exif, if available
    /*
    try {
      if (allowExifThumbnail) {
        ExifManager exifManager = new ExifManager();
        exifManager.readMetadata( imageFile );

        Image preview = exifManager.getThumbnail();
        if (preview != null) {
          image = ImageUtils.getBufferedImage( preview );
        }
      }
    }
    catch( Throwable t ) {
      t.printStackTrace();
      ImageUtils.msg(  
        "-I- createThumbnailImage: no preview for '"+imageFile+"'" );
    }
    */
    
    // try to read the image (unless we already have a valid image)
    try { 
      if (image == null) {
        Image tmp = ImageUtils.loadImage( imageURL );
        image     = ImageUtils.getBufferedImage( tmp );
      }
      if (image == null) {
        ImageUtils.msg( "-E- createThumbnailImage failed: '"+imageURL + "'" );
        return null; // happens when not an image file
      }
    }
    catch( Throwable t ) {
      t.printStackTrace();
      return null;
      // return createErrorThumbnail( imageURL );
    }

    return paintScaledThumbnail( image );
  }



  /**
   * returns a thumbnail image of (size)x(size) pixel by rescaling the
   * given (non-null!) master image.
   */
  public BufferedImage paintScaledThumbnail( BufferedImage image ) { 
    int iw = image.getWidth( null );
    int ih = image.getHeight( null );

    if ((iw <= 0) || (ih <= 0)) { // happens when AWT busy fetching the
                                  // same image...
      return null;
    }

    // Only wanted the thumbnail, don't want to filling it up to 200,200.
    //BufferedImage thumbnail = new BufferedImage( 
    //                                size, 
    //                                size,
    //                                BufferedImage.TYPE_INT_RGB );
    //Graphics g = thumbnail.getGraphics();
    //g.setColor( backgroundColor );
    //g.fillRect( 0, 0, size+1, size+1 );

    //int  xoffset = 0;
    //int  yoffset = 0;
    int  xsize   = 0;
    int  ysize   = 0;
    double scale = 1.0;

    if (iw >= ih) {
      scale = 1.0 * size / iw;
      xsize = (int) Math.round( scale*iw );
      ysize = (int) Math.round( scale*ih );
       
      //xoffset = 0;
      //yoffset = (size - ysize) / 2;
    }
    else { // ih > iw 
      scale = 1.0 * size  / ih;
      xsize = (int) Math.round( scale*iw );
      ysize = (int) Math.round( scale*ih );

      //xoffset = (size - xsize) / 2;
      //yoffset = 0;
    }
   
    // ADDED: Only wanted the thumbnail.
    BufferedImage thumbnail = new BufferedImage( 
                                    xsize, 
                                    ysize,
                                    BufferedImage.TYPE_INT_RGB );
    Graphics g = thumbnail.getGraphics();

if (debug) {
System.out.println( "-#- " + iw + "x" + ih + "  " + scale + " " + "xoffset" + " " + "yoffset" + "   " + xsize + " " + ysize );
}
    RenderingHints hints = new RenderingHints(null);
    hints.put(RenderingHints.KEY_RENDERING,
              RenderingHints.VALUE_RENDER_QUALITY);
    hints.put(RenderingHints.KEY_INTERPOLATION,
              RenderingHints.VALUE_INTERPOLATION_BILINEAR);

    ((Graphics2D)g).setRenderingHints( hints );

    /* Only wanted the thumbnail so don't draw the offset.
    g.drawImage( image, 
                 xoffset, yoffset, xoffset+xsize, yoffset+ysize, // destination
                 0, 0, iw, ih,                   // source image
                 null );        // observer
    */

    g.drawImage( image, 
                 0, 0, xsize, ysize, // destination
                 0, 0, iw, ih,                   // source image
                 null ); 

    image.flush();

    ImageUtils.msg( "-#- ok." );
    return thumbnail;
  }



  /**
   * write the given image to the given file.
   * Uses ImageIO.write().
   */
  public void writeThumbnail( BufferedImage thumbnail, String filename ) {
    try {
      if (thumbnail == null) return;
      ImageIO.write( thumbnail, "jpg", new File(filename) );
    }
    catch( Throwable t ) {
      t.printStackTrace();
      return;
    }
  }


  /**
   * create thumbnails for all (image) file in the given directory.
   * This can take a long time, and should be called in a separate
   * worker thread.
   */
  public void checkCreateThumbnailsForDirectory( File dir ) {
    try {
      if (dir == null) return;
      if (!dir.exists()) return;
      if (!dir.isDirectory()) return;

      String[] names= dir.list();
      for( int i=0; i < names.length; i++ ) {
        File tmp = new File( dir, names[i] );
        if (tmp.isDirectory()) continue;
        // createThumbnailForImageFile( tmp );
        BufferedImage dummy = checkCreateThumbnail( tmp.getPath(), null );
      }
    }
    catch( Exception e ) {
      e.printStackTrace();
    }
  }


  public String getDirectoryName( String path ) {
    int ix = path.lastIndexOf( File.separator );
    if (ix >= 0) return path.substring( 0, ix );
    else         return "";
  }


  /**
   * calculate a hash code for a given string.
   * This method is used by getThumbnailsDirectoryName() to create 
   * the (short) directory names used to store the thumbnail images.
   */
  public int getHashForString( String s ) {
    int hashCode = 0;
    int limit = s.length();
    for (int i = 0; i < limit; i++)
      hashCode = hashCode * 31 + s.charAt(i);
    return hashCode;
  }


  private final static String hex = "0123456789abcdef";


  /**
   * return a (short) directory name used to store the thumbnail images
   * for the given source directory name.
   * The returned value currently takes the following form:
   * {niffler.home}/thumbnails/hashcode/
   */
  public String getThumbnailsDirectoryName( String dirname ) {
    StringBuffer sb = new StringBuffer();
    sb.append( thumbnailsBaseDirectoryName );
    sb.append( File.separator );

    int hashcode = getHashForString( dirname );
    for( int i=0; i < 8; i++ ) {
      int ix = hashcode & 0xf; 
      hashcode = hashcode >>> 4;
      sb.append( hex.charAt(ix) );
    }

    String tmp = Integer.toHexString( hashcode );
    return sb.toString();
  }


  public String getThumbnailsBaseDirectoryName() {
    return SetupManager.getNifflerDirectory()
         + File.separator
         + "thumbnails";
  }


  public boolean isInsideThumbnailsDirectory( File dir ) {
    String path = dir.getPath();
    if (path.startsWith( thumbnailsBaseDirectoryName )) return true;
    else                                                return false;
  }


  public class BackgroundLoader extends Thread {
    private int DEFAULT_SLEEP_MILLIS = 200;
    private Vector filenameQueue;
    private Vector repaintTargetQueue;
    private String lock = "lock";


    public BackgroundLoader() {
      filenameQueue = new Vector();
      repaintTargetQueue = new Vector();
      setPriority( Thread.MIN_PRIORITY );
      setDaemon( true );
    }


    public void addThumbnailOrder( String imagePathName, JComponent target ) {
      synchronized (lock) {
        Object status = cache.get( imagePathName );

        filenameQueue.insertElementAt( imagePathName, 0 );
        repaintTargetQueue.insertElementAt( target, 0 );

        if ("ONE_FAILURE".equals(status)) { 
          ;
        }
        else { // not cached yet, order the thumbnail
          cache.put( imagePathName, "BUSY" );
        }
      }

// FIXME: THIS JUST IS BROKEN 
      // this.interrupt(); // THIS ALSO KILLS ONGOING AWT IMAGE LOADING
    }


    public int countPendingOrders() {
      return filenameQueue.size();
    }


    public void run() {
      while( true ) {
        try {

          String   filename = null;
          JComponent target = null;
          BufferedImage tmp = null;

          synchronized (lock) {
            if (filenameQueue.size() > 0) {
              filename = (String) filenameQueue.elementAt(0);
              target   = (JComponent) repaintTargetQueue.elementAt(0);

              filenameQueue.removeElementAt(0);
              repaintTargetQueue.removeElementAt(0);
            }

            checkCacheSizeLimit();
          }

          if (filename != null) {

            // note that this method already updates the cache when successful
            tmp = checkCreateThumbnail( filename, null );

            if (tmp != null) { // we got that thumbnail
              ImageUtils.msg( "-#- BackgroundLoader: loaded '" 
                          + filename + "' + " + tmp.hashCode() );
            }
            else {
              ImageUtils.msg( "-W- BackgroundLoader: FAILED '" 
                          + filename + "'" );

              Object status = cache.get( filename );
              if ("ONE_FAILURE".equals( status )) 
                   cache.put( filename, "ERROR" );
              else cache.put( filename, "ONE_FAILURE" );
            }

            if (target != null) {
              target.repaint( 200 );
            }
          }
          else {
            Thread.sleep( DEFAULT_SLEEP_MILLIS );
          }

        }
        catch( InterruptedException ie ) {
          ; // ignore
        }
        catch( Throwable t ) {
          t.printStackTrace();
        }

      }
    } // run

  } // BackgroundLoader







  public static String usage() {
    return "Usage: java ThumbnailManager <imagedir>\n"
    + "creates niffler thumbnails for all images in the specified directory\n"
    + "and stores them in the thumbnails directory, currently\n"
    + "<user.home>/.niffler/thumbs.\n"
    + "Note: this process can take a few seconds.";
  }


  public static void main( String args[] ) throws Exception {
    if (args.length == 0) {
      System.out.println( usage() );
      System.exit( 0 );
    }

    ThumbnailsManager tm = new ThumbnailsManager();
    tm.checkCreateThumbnailsForDirectory( new File( args[0] ) );
  }
}
