/* ThumbnailsPanel.java - class niffler.ThumbnailsPanel
 * 
 * 07.07.06 - return (0,0) as preferred size when not enabled
 * 21.06.06 - setEnabled stuff
 * 02.03.06 - new method getThumbnailBounds for scrollToVisible 
 * 18.01.06 - some finetuning
 * 17.01.06 - new class
 *
 * (C) 2006, Norman Hendrich
 * All rights reserved. See file "niffler/license.txt" for details.
 */

/*
 * Niffler Image Viewer
 * Copyright (c) 2003-2006, Norman Hendrich
 * All rights reserved.
 *
 * Contact:
 *
 * Norman Hendrich
 * c/o University of Hamburg
 * FB Informatik
 * Vogt-Koelln-Str. 30
 * D 22527 Hamburg
 * Germany
 *
 * hendrich@informatik.uni-hamburg.de
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions 
 * are met:
 *
 *   * Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above copyright notice, 
 *     this list of conditions and the following disclaimer in the documentation 
 *     and/or other materials provided with the distribution.
 *   * Neither the name of the University of Hamburg nor the names of its 
 *     contributors may be used to endorse or promote products derived from 
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE 
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER 
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, 
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/**
 * Jun 2007
 * modified by Cristina Roura Claver cristina.roura@gmail.com
 * 
 * CHANGES:
 *      Package name modified.
 * 	set a background color.
 *	changed defaults, rows = 1, spacing = 0, thumbnail size = 200,200
 *	in method paintComponent, don't fill.
 *
 */

package lhmc.gui.thumbnail;

import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import java.io.*;
import java.util.*;
import javax.imageio.*;
import javax.swing.*;


/**
 * Swing GUI component to layout and display image thumbnails.
 */
public class ThumbnailsPanel extends JPanel {
  
  private boolean debug = false;
  private boolean enabled = true;

  private String[]        filenames;
  private BufferedImage[] thumbnails;
  private int activeImageIndex = -1;

  private int thumbnailWidth;
  private int thumbnailHeight;
  private int spacing;
  private int n_thumbs_per_row;

  private ThumbnailsManager manager;

  private Color thumbnailBackground;
  private Color panelBackground;

  public final static int OUT_OF_RANGE = -1;
  
  private static final Color backgroundColor = new Color(242,242,242);


  /**
   * creates a new empty ThumbnailsPanel and initialize the GUI defaults.
   */
  public ThumbnailsPanel() {
    super();
 
    thumbnails = new BufferedImage[0];
    filenames  = new String[0];
    initializeDefaults();
  }
  
  public void initializeDefaults() {
    n_thumbs_per_row = 1;
    spacing = 0;
    thumbnailWidth = 200;
    thumbnailHeight = 200;
    thumbnailBackground = Color.BLACK; 
    panelBackground = new Color( 200, 200, 200 );

    setEnabled( SetupManager.getBoolean( "niffler.enableThumbnailSupport", true ));
  }

  public void setEnabled( boolean b ) {
    enabled = b;
    repaint();
  }


  public boolean isEnabled() {
    return enabled;
  }

  /**
   * set the actual thumbnail images to be displayed by this panel.
   */
  public void setThumbnails( BufferedImage[] thumbs ) {
    thumbnails = thumbs;
    filenames  = new String[thumbs.length];
    invalidate();
  }


  public void setThumbnails( String _filenames[], ThumbnailsManager _manager ) {
    filenames  = _filenames;
    thumbnails = new BufferedImage[filenames.length];
    manager    = _manager;
  }


  /**
   * note: this method does not request a repaint; you will have to do that
   * separately.
   */
  public void setActiveImageIndex( int i ) {
    activeImageIndex = i;
  }


  /**
   * set the width and height to be used for each thumbnail image.
   * Typical values are 80x80 or 100x100 pixels.
   */
  public void setThumbnailDimension( Dimension d ) {
    thumbnailWidth = d.width;
    thumbnailHeight = d.height;
    invalidate();
  }

  /**
   * returns the current screen size reserved for a single thumbnail image.
   */
  public Dimension getThumbnailDimension() {
    return new Dimension( thumbnailWidth, thumbnailHeight );
  }


  /**
   * returns the extra spacing pixels reserved between thumbnails.
   */
  public int getThumbnailSpacing() {
    return spacing;
  }


  /**
   * set the number of thumbnails to be displayed in one row.
   */
  public void setPreferredNumberOfThumbnailsPerRow( int n ) {
    n_thumbs_per_row = n;
    invalidate();
  }


  /**
   * calculates the preferred size from 'n_thumbs_per_row' and the current
   * number of thumbnails.
   * 
   * FIXME: it might be better to first calculate the value of
   * 'n_thumbs_per_row' to fit the available screen space.
   */
/*  public Dimension getPreferredSize() {
    if (!isEnabled()) return new Dimension(0,0);

    int    w = spacing + n_thumbs_per_row * (thumbnailWidth + spacing);
    int rows = (int) Math.ceil( 1.0 * thumbnails.length / n_thumbs_per_row );
    int    h = spacing + rows * (thumbnailHeight + spacing);
    return new Dimension( w, h );
  }


  public Dimension getMinimumSize() {
    return new Dimension(0,0);
    // return getPreferredSize();
  }


  public Rectangle getThumbnailBounds( int index ) {
    int row = index / n_thumbs_per_row;
    int col = index - (row * n_thumbs_per_row);

    // ImageUtils.msg( "-X- " + index + " " + row + " " + col );

    int   x = 0 + col*(thumbnailWidth+spacing); // don't include left spacing
    int   y = 0 + row*(thumbnailHeight+spacing); // don't include top spacing
    int   w = thumbnailWidth + spacing;
    int   h = thumbnailHeight + spacing;

    Rectangle rect = new Rectangle( x, y, w, h );

    // ImageUtils.msg( "-X- ThumbnailsPanel.getThumbnailBounds " + index + " " + rect );
    return rect;
  }

*/

  /**
   * returns the index of the thumbnail at the given position
   * or -1 if outside the valid range.
   */
  /*
  public int getThumbnailIndexForPosition( Point p ) { 

    int col = (p.x - spacing) / (thumbnailWidth+spacing);
    int row = (p.y - spacing) / (thumbnailHeight+spacing);

    if (col < 0)                return OUT_OF_RANGE;
    if (col > n_thumbs_per_row) return OUT_OF_RANGE;
    if (row < 0)                return OUT_OF_RANGE;

    int index = row*n_thumbs_per_row + col;
    if (index >= thumbnails.length) return OUT_OF_RANGE;
    else                            return index;
  }
*/

  public void paintComponent( Graphics g ) {
      
    //System.out.println( "ThumbnailsPanel - paintComponent: entra");

    long t1 = System.currentTimeMillis();
    int n_scaled = 0;
    int n_unscaled = 0;
    int n_skipped = 0;

    // uniform background
    // 
    //Dimension d = getSize();
    //g.setColor( panelBackground );
    //g.fillRect( 0, 0, d.width+1, d.height+1 );

    if (!enabled) {
      g.setColor( Color.black );
      g.drawString( "thumbnails disabled", 20, 30 );
      return;
    }

    // loop to paint the thumbnails
    // 
    Rectangle clip = g.getClipRect();
    for( int i=0; i < thumbnails.length; i++ ) {
      try {
        //System.out.println("ThumbnailsPanel - fileName: " + filenames[i]);
        int row = 1;
        int col = 1;

        // upper left and lower right corner of the current thumbnail
        // 
        int x0  = 0;
        int y0  = 0;
        int x1  = x0 + thumbnailWidth+spacing;
        int y1  = y0 + thumbnailHeight+spacing;

        // check the current image bounding box vs. the current clip
        // rectangle, and skip image rendering when outside the clip. 
        // As the measured performance gain of this optimiziation is
        // very small with JDK 1.5, it seems that the JDK already uses
        // a similar optimization for Graphics.drawImage()...
        // 
        if (   (x0 > (clip.x+clip.width)) 
            || (x1 < clip.x)
            || (y1 < clip.y) 
            || (y0 > (clip.y+clip.height)))
        {
          n_skipped ++;
          continue;
        }

        // first render the background for the current thumbnail
        // 
        //g.setColor( thumbnailBackground );
                // g.fillRect( x0-1, y0-1, thumbnailWidth+2, thumbnailHeight+2 );
        //g.fillRect( x0, y0, thumbnailWidth, thumbnailHeight );
        if (i == activeImageIndex) {
          g.setColor( Color.BLUE );
          g.drawRect( x0-1, y0-1, thumbnailWidth+2, thumbnailHeight+2 );
        }

        // second, the thumbnail image itself; scaled down to fit if necessary.
        // Skip null images.
        //
        Image image = thumbnails[i];
        if (image == null) { // try to load via manager
          if (manager != null) {
            image = manager.getCachedThumbnailOrNull( filenames[i], this );
          }
        }
        if (image == null) {
          // still no thumbnail (yet). Draw the filename instead
          // 
          //File    file = new File( filenames[i] );
          //String fname = file.getName(); // strips off the directories part

          // System.out.println( "-W- no thumbnail yet: " + file );

/*          Graphics gg = g.create();
          gg.clipRect( x0, y0, thumbnailWidth+1, thumbnailHeight+1 );
          gg.setColor( Color.lightGray ); // orange );
          gg.fillRect( x0, y0, thumbnailWidth+1, thumbnailHeight+1 );
          gg.setColor( Color.black );
          gg.drawString( fname, x0+5, y0+thumbnailWidth/2 );
          gg.dispose();
*/
          continue; // next loop iteration
        }

        int tw  = image.getWidth( null );
        int th  = image.getHeight( null );

        if ((tw > thumbnailWidth) || (th >thumbnailHeight)) { // scale
          double scalex = 1.0 * tw / thumbnailWidth;
          double scaley = 1.0 * th / thumbnailHeight;
          double scale  = Math.max( scalex, scaley );
          int sw = (int) (tw/scale);
          int sh = (int) (th/scale);

          int dx = (int) (thumbnailWidth - sw) / 2;
          int dy = (int) (thumbnailHeight - sh) / 2;

          g.drawImage( image,    // current thumbnail
                       x0+dx, y0+dy, x0+dx+sw, y0+dy+sh, // dest coords
                       0, 0, tw, th,                     // source coords
                       null );                           // observer 
          n_scaled ++;
        }
        else { // thumbnail fits, center image
          int dx = (thumbnailWidth  - tw) / 2;
          int dy = (thumbnailHeight - th) / 2;
          g.drawImage( image, x0+dx, y0+dy, null ); // null observer
          n_unscaled ++;
        }

      }
      catch( Exception e ) {
        System.err.println( "-E- could not render thumbnail[" + i + "]" );
        e.printStackTrace();
      }
      catch( Throwable t ) {
        System.err.println( "-E- could not render thumbnail[" + i + "]" );
        t.printStackTrace();
      }
    } // for i 

    if (debug) {
      long t2 = System.currentTimeMillis();
      System.out.println( "-#- ThumbnailPanel.paintComponent took " 
                        + (t2-t1) + " msecs. "
                        + " scaled=" + n_scaled 
                        + " original size=" + n_unscaled 
                        + " skipped=" + n_skipped
                        );
    }
  } // paintComponent

/*  
    public void createThumbnailsPanel() {
        for( int i=0; i < thumbnails.length; i++ ) {
            Image image = thumbnails[i];
            if (image == null) { // try to load via manager
                if (manager != null) {
                    image = manager.getCachedThumbnailOrNull( filenames[i], this );
                }
            }
            if (image == null) {
                // still no thumbnail (yet). Draw the filename instead
                
                File    file = new File( filenames[i] );
                String fname = file.getName(); // strips off the directories part
        
                //XXX FER THUMBNAIL BUIT?
        
                continue; // next loop iteration
            }
        
            ThumbnailJComponent thumbnail = new ThumbnailJComponent(image, thumbnailWidth, thumbnailHeight );
        
            this.add(thumbnail);

        }
   }
   
   */
}