/*
 * ThumbnailFacade.java
 *
 * Copyright 2007 Cristina Roura Claver cristina.roura@gmail.com
 *
 * This file is part of LHMC.
 *
 * LHMC is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * LHMC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package lhmc.gui.thumbnailsbrowser;

import javax.swing.*;
import java.io.File;
import java.awt.*;

import javax.imageio.*;
import java.awt.image.*;
import java.io.*;
import java.util.*;
import java.net.*;

import lhmc.files.*;
import lhmc.gui.thumbnail.*;
import lhmc.gui.util.*;
import lhmc.utils.*;

/**
 * ThumbnailFacade provides a simplified interface to the external classes:
 * ThumbanailsManager, ThumbnailsPanel.
 * Manages the files from a directory and its thumbnails.
 *
 * @author 	Cristina Roura Claver
 * @version 	0.01 Jun 2007
 */
public class ThumbnailFacade {
    
    static int FILE_TYPE = 0;
    static int SMBFILE_TYPE = 1;
    
    /* A manager that keeps a cache memory of the thumbnails created */
    private ThumbnailsManager  thumbnailsManager;
    
    /* It maintains the current directory tree node */    
    private FileManager fileManager;
    
    private ConfigReader config = new ConfigReader();
    
    /**
     * Constructs the thumbnail facade.
     * @param files an array of files in a directory.
     */    
    public ThumbnailFacade(URL[] files) {
        thumbnailsManager = new ThumbnailsManager();
        fileManager = new FileManager(files);
        
    }

    /**
     * Changes the current directory, changin the tree node
     * @param files The new directory files.
     */        
    public void changeDirectory(URL[] files) {
        fileManager.changeTreeNode(files);
    }

    /**
     * Gets the file name at the certain position.
     * @param index The position of the thumbnail.
     * @return      The file name.
     */            
    public String getFileNameAt( int index ) {
         String fileName = fileManager.getFileNameAt(index);
         return fileName;        
    }
    
    /**
     * Gets the file at the certain position.
     * @param index The position of the thumbnail file.
     * @return      The url file.
     */            
    public URL getFileAt( int index ) {
       return fileManager.getFileAt(index);
    }    

    /**
     * Creates a thumbnail panel for one file at certain position in the tree.
     * @param index The position of the file.
     * @return      The thumbnail panel.
     */         
    public JPanel createThumbnail(int index) {
        URL file = getFileAt(index);
        boolean isDirectory = fileManager.isDirectoryNode(index);
        
        ImageFileFilter imageFileFilter = new ImageFileFilter();
        AudioFileFilter audioFileFilter = new AudioFileFilter();
        VideoFileFilter videoFileFilter = new VideoFileFilter();
        
        /* if its an image, makes a thumbnail from the image */
        if (imageFileFilter.accept(file) && !isDirectory) {  
            //System.out.println("CREATE THUMBNAIL: Its a image or directory: " + file);
            String[] name = new String[1];
            name[0] =  getFileNameAt(index);
            
            /* Using ThumbnailsPanel, a class that creates a panel from an image file. */
            ThumbnailsPanel thumbPane = new ThumbnailsPanel();
            thumbPane.setThumbnails(name, thumbnailsManager );
            return  thumbPane;            
        /* if its an audio file, makes a thumbnail from the default audio image */
        } else if (imageFileFilter.accept(file) && isDirectory) { 
            //System.out.println("CREATE THUMBNAIL: Its an audio: " + file);
            /* Using ThumbnailsPanel, a class that creates a panel from an image file. */
            ThumbnailsPanel audioPane = new ThumbnailsPanel();
            BufferedImage[] img = new BufferedImage[1];
            try {
                String str = config.getProperty("IconDirectory");   
                img[0]= ImageIO.read(new File(str));
            } catch (IOException e) {}
           audioPane.setThumbnails(img);
           return audioPane;
        } else if (audioFileFilter.accept(file)) {
            //System.out.println("CREATE THUMBNAIL: Its an audio: " + file);
            /* Using ThumbnailsPanel, a class that creates a panel from an image file. */
            ThumbnailsPanel audioPane = new ThumbnailsPanel();
            BufferedImage[] img = new BufferedImage[1];
            try {
                String str = config.getProperty("IconAudioFile");   
                img[0]= ImageIO.read(new File(str));
            } catch (IOException e) {}
           audioPane.setThumbnails(img);
           return audioPane;
        }  else if (videoFileFilter.accept(file)) {
            
            /* Using ThumbnailsPanel, a class that creates a panel from an image file. */
            //mplayer video.avi -ss 60 -frames 1 -vo jpeg                        
            //String[] command = new String[] {"/usr/bin/mplayer", file.getAbsolutePath(), "-nosound", "-ss", "30", "-frames", "2", "-vo", "png"};        
            //try {
            //    Process playerPrc = Runtime.getRuntime().exec(command);
            //} catch(Exception e) {
            //    throw new RuntimeException("could not exec \"" + command + "\": " + e);
            //}
            
            ThumbnailsPanel videoPane = new ThumbnailsPanel();
            BufferedImage[] img = new BufferedImage[1];
            try {                
                String str = config.getProperty("IconVideoFile");    
                img[0]= ImageIO.read(new File(str));
            } catch (IOException e) {}
            
           videoPane.setThumbnails(img);
           return videoPane;
        } else {
            return new JPanel();
        }
    }         
}
