/*
 * PluginFactory.java	
 *
 * Copyright 2007 Cristina Roura Claver cristina.roura@gmail.com
 *
 * This file is part of LHMC.
 *
 * LHMC is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * LHMC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package lhmc.plugins;

import javax.activation.MimetypesFileTypeMap;
import java.io.File;
import java.io.IOException;
import java.net.*;

import jcifs.smb.*;

import lhmc.files.*;
        
/**
 * PluginFactory decides which plugin must be executed depending
 * on the file type of the giben file.
 *
 * @author 	Cristina Roura Claver
 * @version 	0.01 29 Mar 2007
 */
public class PluginFactory {
    
    static final int IMAGE_TYPE = 1;
    static final int MUSIC_TYPE = 2;
    static final int MOVIE_TYPE = 3;
    static final int AUDIO_CD_TYPE = 4;    
    static final int VIDEO_DVD_TYPE = 5;    
    static final int WATCH_TV_TYPE = 6;    
    static final int RECORD_TV_TYPE = 7;
    /**
     * Returns the plugin suitable to execute the giben file.
     * @param url  the url file to execute.
     * @return      the plugin to use for executing the file.
     */
    public Plugin getPlugin( URL url ) {
        
        jcifs.Config.registerSmbURLHandler();    
        String protocol = url.getProtocol();
	File file = null;
	SmbFile smbfile = null;
                
        int fileType = -1;
        int i = 0;
        
        if (protocol.equals("file")) {
            file = new File(url.getPath());
            /* if it is a directory get the type of the first file that's not a directory */     
            if (file.isDirectory()) { 
                /* Array of files */
                File[] files = file.listFiles();
                while (files[i].isDirectory()) {
                    i++;
                }
                fileType = figureOutfileType(files[i]);
            } else {
                fileType = figureOutfileType(file);
            }  
            
        } else if (protocol.equals("smb")) {
            try {
                smbfile = new SmbFile("smb:/" + url.getPath());        
                /* if it is a directory get the type of the first file that's not a directory */     
                if (smbfile.isDirectory()) { 
                    /* Array of files */
                    SmbFile[] smbFiles = smbfile.listFiles();
                    while (smbFiles[i].isDirectory()) {
                        i++;
                    }
                    fileType = figureOutfileType(smbFiles[i]);
                } else {
                    fileType = figureOutfileType(smbfile);
                }        
            } catch(MalformedURLException e) {
                System.out.println("PluginFactory - getPlugin()");
                System.out.println("MalformedURLException " + e);
            } catch(SmbException e) {
                System.out.println("SmbException " + e);
            }                   
        }        
                      	    	
    	switch (fileType) {
	    case IMAGE_TYPE:
        	return new ImagePlugin();		
	    case MUSIC_TYPE:
	       	return new BasicPlayerPlugin();
            case MOVIE_TYPE:
	       	return new MPlayerPlugin();
            default:
                System.out.println("PluginFactory - ERROR: Case failed");
		return null;
        }
    }
    
    /**
     * Returns the plugin suitable to execute the giben cdType.
     * @param cdType  the cdType to execute.
     * @return      the plugin to use for executing the cd or dvd.
     */
    public Plugin getPlugin( int cdType ) {
        
    	switch (cdType) {
	    case AUDIO_CD_TYPE:
        	//return new CddaPlayerPlugin();		
        	return new CddaMPlayerPlugin();                
	    case VIDEO_DVD_TYPE:
                //return new DvdMPlayerPlugin();
                return new DvdXinePlugin();              	       	
            case WATCH_TV_TYPE:
		return new DvbMPlayerPlugin();  
               //return new TvMPlayerPlugin();  
	    case RECORD_TV_TYPE:
		return new MencoderPlugin();
            default:
                System.out.println("PluginFactory - ERROR: Case failed");
		return null;
        }
    }    

    /**
     * Figures out the type of file.
     * @param file  the file to get the type.
     * @return      an integer that matches d.
     */    
    public int figureOutfileType(File file) {            
        ImageFileFilter imageFileFilter = new ImageFileFilter();
        AudioFileFilter audioFileFilter = new AudioFileFilter();
        VideoFileFilter videoFileFilter = new VideoFileFilter();
        
        /* if its an image, makes a thumbnail from the image */
        if (imageFileFilter.accept(file)) {  
            return IMAGE_TYPE; 
        /* if its an audio file, makes a thumbnail from the default audio image */
        } else if (audioFileFilter.accept(file)) {
            return MUSIC_TYPE;
        }  else if (videoFileFilter.accept(file)) {
            return MOVIE_TYPE;
        } else {
           return 0;
        }        
    } 

    /**
     * Figures out the type of file.
     * @param smbFile  the smbfile to get the type.
     * @return      an integer that matches d.
     */        
    public int figureOutfileType(SmbFile smbFile) {            
        ImageFileFilter imageFileFilter = new ImageFileFilter();
        AudioFileFilter audioFileFilter = new AudioFileFilter();
        VideoFileFilter videoFileFilter = new VideoFileFilter();
        
        /* if its an image, makes a thumbnail from the image */
        if (imageFileFilter.accept(smbFile)) {  
            return IMAGE_TYPE; 
        /* if its an audio file, makes a thumbnail from the default audio image */
        } else if (audioFileFilter.accept(smbFile)) {
            return MUSIC_TYPE;
        }  else if (videoFileFilter.accept(smbFile)) {
            return MOVIE_TYPE;
        } else {
           return 0;
        }        
    }
}
