/*
 * ScaledImageJComponent.java	
 *
 * Copyright 2007 Cristina Roura Claver cristina.roura@gmail.com
 *
 * This file is part of LHMC.
 *
 * LHMC is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * LHMC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package lhmc.plugins;
        
import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import java.io.*;
import javax.imageio.*;
import javax.swing.*;
import java.awt.geom.*;
import java.net.*;

import jcifs.smb.*;

import lhmc.files.*;

/**
 * ScaledImageJComponent reads an image file and resizes it to fit fullscreen
 * mantaing its ratio.
 * Image I/O has built-in support for GIF, PNG, JPEG, BMP, and WBMP.
 *
 * @author 	Cristina Roura Claver
 * @version 	0.01 Jun 2007
 */
public class ScaledImageJComponent extends JComponent {
      
    private BufferedImage img;
       
    int screenWidth;        /* Display width */
    int screenHeight;       /* Display height */
    
    static int LOCAL_FILE = 0;
    static int REMOTE_SMBFILE = 1;     
    
    /**
     * Constructs a null scaled image.
     */    
    public ScaledImageJComponent() {
        img = null;
        getScreenSize();
    }

    /**
     * Constructs a scaled image from a file.
     * @param file  the url image to read.
     */        
    public ScaledImageJComponent(URL file) {
        //System.out.println("ScaledImageJComponent - ScaledImageJComponent() - Url file: " + file);
	readImage(file);
        getScreenSize();
    }

    /**
     * Constructs a scaled image from a string path.
     * @param str   the srt of the image to read.
     */            
    public ScaledImageJComponent(String str) {
        try {
            readImage(new URL(str));
            getScreenSize();
        } catch(MalformedURLException e) {
                System.out.println("MalformedURLException " + e);
        }
    }

    /**
     * Creates a scaled image from a strig path.
     * @param str  The path of the image.
     */     
    public void setImage(String str) {
        try {
            readImage(new URL(str));
        } catch(MalformedURLException e) {
                System.out.println("MalformedURLException " + e);
        }            
    }

    /**
     * Creates the image from an url file.
     * @param file  The image url file.
     */   
    private void readImage(URL file) {
        try {                
            UrlUtils urlUtils = new UrlUtils(file);
            if (urlUtils.getLocation() == REMOTE_SMBFILE) {
                img = ImageIO.read(new SmbFileInputStream(urlUtils.getSmbFile(file)));
            } else if (urlUtils.getLocation() == LOCAL_FILE) {
                img = ImageIO.read(file);
            }       
	} catch (IOException e) { 
            System.out.println("IOException " + e);
        }             
        /*
        try {
            jcifs.Config.registerSmbURLHandler();    
            String protocol = file.getProtocol();
            System.out.println("ScaledImageJComponent - readImage() - Url file: " + file);                
            if (protocol.equals("smb")) {
                img = ImageIO.read(new SmbFileInputStream(new SmbFile("smb:/" + file.getPath())));
            } else {
                img = ImageIO.read(file);
            }
	} catch (IOException e) { 
            System.out.println("IOException " + e);
        }  
         **/      
    }  
    
    /**
     * Gets and sets the screen size.
     */      
    private void getScreenSize() {   
        GraphicsDevice device = GraphicsEnvironment.getLocalGraphicsEnvironment().getDefaultScreenDevice();        
        DisplayMode mode = device.getDisplayMode();
         
        screenWidth = mode.getWidth();
        screenHeight = mode.getHeight();
    }

    /**
     * Gets the preferred size.
     * @return     The screen size.
     */          
    public Dimension getPreferredSize() {
        if (img == null) {
             return new Dimension(screenWidth,screenHeight);
        } else {
            return new Dimension(screenWidth, screenHeight);
       }
    }

    /**
     * Invoques the paintComponent method.
     * @param g the Graphics context.
     */        
    public void paintScaledImage(Graphics g) {
        paintComponent(g);
    }

    /**
     * Reimplementation of paintComponent to paint the img.
     * @param g the Graphics context.
     */           
    protected void paintComponent(Graphics g) {
        /* black background */             
        g.setColor( Color.black);
        g.fillRect( 0, 0, screenWidth, screenHeight );    

        /* if screenImage is not null (image loaded and ready) */
        if (img != null) {
            float imgWidth, imgHeight;
            float newWidth, newHeight;
            float imgRatio, scrRatio;

            /* gets image and frame size */
            imgWidth = img.getWidth(this);
            imgHeight = img.getHeight(this);
 
            
            /* calculates aspect ratio */
            imgRatio = imgHeight / imgWidth;
            scrRatio = (float)screenHeight / (float)screenWidth;
            
            /* the new size */
            if( imgRatio < scrRatio ) {
                newWidth = screenWidth;
                newHeight = imgHeight * (newWidth/imgWidth);
                              
            } else {
                
                newHeight = screenHeight;
                newWidth = imgWidth * (newHeight/imgHeight);                          
            }
            
            int centerX = ((screenWidth - (int)newWidth) / 2);
            int centerY = ((screenHeight - (int)newHeight) / 2); 
            
            /* draws the image */
            g.drawImage(img, centerX, centerY, (int)newWidth, (int)newHeight, null);
        }
    }
}

